#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for Plex Upgrade
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse
from argcomplete.completers import EnvironCompleter     # used for file/path arguments

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.plex_upgrade import get_available_plex_server_package_name
from media_mgr.plex_upgrade import show_running_and_available_plex_server_versions
from media_mgr.plex_upgrade import download_new_plex_server_archive

from media_mgr.plex_upgrade import upgrade_plex_server, upgrade_all_plex_servers

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}PLEX Upgrade General Purpose {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action='store_true',
                            help='run with verbosity hooks enabled')

        parser.add_argument('--version', action="store_true", help='top-level package version')

        subparsers = parser.add_subparsers(dest='cmd')

        p_show_avail_installer = subparsers.add_parser('show.avail.installer',
                help='show available PLEX server binary')

        p_plex_version = subparsers.add_parser('plex.ver',
                help='show running and available PLEX server versions')
        p_plex_version.add_argument('--ipv4', metavar='<server.addr>',
                default=None, help='PLEX Server IPV4')

        p_download = subparsers.add_parser('download.bin',
                help='download new PLEX server binary')
        p_download.add_argument('--ipv4', metavar='<server.addr>',
                default=None, help='PLEX Server IPV4')
        p_download.add_argument('--dir', default='/tmp',
                metavar='<dwnld.path>', help='PLEX bin download path').completer = EnvironCompleter

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'show.avail.installer':
            get_available_plex_server_package_name(verbose = True)

        elif args.cmd == 'plex.ver':
            show_running_and_available_plex_server_versions(ipv4 = args.ipv4,
                    debug = args.verbose)

        elif args.cmd == 'download.bin':
            download_new_plex_server_archive(ipv4 = args.ipv4,
                    downloadDir = args.dir, debug = args.verbose)

    except Exception as e:
        exception_details(e, "Plex Upgrade Utility CLI", raw=True)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_plex_upgrade():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}PLEX Server Upgrade {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action='store_true',
                help='run with verbosity hooks enabled')

        parser.add_argument('--ipv4', metavar='<server.addr>',
                default=None, help='PLEX Server IPV4')
        parser.add_argument('--dir', default='/tmp',
                metavar='<dwnld.path>', help='PLEX bin download path').completer = EnvironCompleter
        parser.add_argument("--force", action='store_true', help='force upgrade even if versions match')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        upgrade_plex_server(ipv4 = args.ipv4, downloadDir = args.dir,
                force = args.force, debug = args.verbose)

    except Exception as e:
        exception_details(e, "Plex Server Upgrade CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_plex_upgrade_all_servers():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}PLEX Server Upgrade ' + \
                            f'{color.CBLUE2}(All Servers) {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action='store_true',
                help='run with verbosity hooks enabled')

        parser.add_argument('--dir', default='/tmp',
                metavar='<dwnld.path>', help='PLEX bin download path').completer = EnvironCompleter
        parser.add_argument("--force", action='store_true', help='force upgrade even if versions match')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        upgrade_all_plex_servers(downloadDir = args.dir, force = args.force,
                debug = args.verbose)

    except Exception as e:
        exception_details(e, "Plex Server Upgrade (All Servers) CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Media EXIF Rename
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

import sys
import os
import json
import time
import datetime
import random
import contextlib


import imghdr   # now dependant on the standard-imghdr package (pip)
import exiv2

from quickcolor.color_def import color
from delayviewer.stopwatch import Stopwatch, handle_stopwatch

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_path_info(fullPathToFile = None):
    if not os.path.isfile(fullPathToFile):
        raise ValueError(f'Error! ->{fullPathToFile}<- is not a file!')

    dirName = os.path.dirname(fullPathToFile)
    baseName = os.path.basename(fullPathToFile)
    fileName, extension = os.path.splitext(fullPathToFile)

    return dirName, baseName, fileName, extension

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_exif_data(fullPathToFile = None):
    if not os.path.isfile(fullPathToFile):
        raise ValueError(f'Error! ->{fullPathToFile}<- is not a file!')

    if imghdr.what(fullPathToFile) not in ['jpeg', 'gif']:
        raise ValueError(f'Error! ->{fullPathToFile}<- is neither a JPEG nor a GIF!')

    # workaround to not having the function calls print:
    # "Directory Canon has an unexpected next pointer; ignored."
    # with contextlib.redirect_stdout(None) and contextlib.redirect_stderr(None):
    with contextlib.redirect_stderr(None):
        image = exiv2.ImageFactory.open(fullPathToFile)
        image.readMetadata()

        exifDict = {}
        for entry in image.exifData():
            entryStrList = entry.__str__().split(f'{entry.key()}:')
            entryStrList = [ s.strip() for s in entryStrList ]
            exifDict[entry.key()] = entryStrList[1]

        return exifDict

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_exif_datetime_original(fullPathToFile = None):
    exifData = get_exif_data(fullPathToFile)

    try:
        dateTimeOrig = exifData['Exif.Photo.DateTimeOriginal'].replace(':', '-')
        dateTimeOrig = dateTimeOrig.replace(' ', '_')
        return dateTimeOrig

    except:
        print(f'{color.CRED2}Warning:{color.CEND} Problem extracting EXIF data ' + \
                f'from {color.CYELLOW}{fullPathToFile}{color.CEND}')
        raise

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

@handle_stopwatch
def rename_all_jpg_in_path_to_exif_dates(pathToJpgs = None, stopwatch = None):
    if not os.path.isdir(pathToJpgs):
        raise ValueError(f'Error! ->{pathToJpgs}<- is not a directory!')

    files = [ f for f in os.listdir(pathToJpgs) if os.path.isfile(os.path.join(pathToJpgs, f)) ]
    if not len(files):
        print(f'{color.CRED2}Warning:{color.CEND} Could not find EXIF media ' + \
                f'in {color.CYELLOW}{pathToJpgs}{color.CEND}')
        return

    errorRenamingTheseFiles = []
    stopwatch.start(f'{color.CBLUE2}-- Attempting to rename {color.CWHITE2}{len(files)}' + \
            f'{color.CBLUE2} file(s) to EXIF datestamps...{color.CEND}', 85)
    for file in files:
        fullPathToFile = pathToJpgs + '/' + file
        try:
            exifData = get_exif_datetime_original(fullPathToFile)
            if exifData:
                fullPathToChangedFile = pathToJpgs + '/' + str(exifData) + '.jpg'
                os.rename(fullPathToFile, fullPathToChangedFile)
        except:
            errorRenamingTheseFiles.append(fullPathToFile)

    if len(errorRenamingTheseFiles):
        stopwatch.stop(f'{color.CRED2}Completed with errors{color.CEND}!')
        print(f'{color.CRED2}Warning:{color.CEND} Could not do anything with the following files:')
        for file in errorRenamingTheseFiles:
            print(f' -- {color.CYELLOW}{file}{color.CEND}')
    else:
        stopwatch.stop()

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_recursive_subdir_list(startPath):
    dirsInDir = [ f'{startPath}/{x}' for x in os.listdir(startPath) if os.path.isdir(f'{startPath}/{x}')]

    if len(dirsInDir):
        for item in dirsInDir:
            moreDirs = get_recursive_subdir_list(startPath = item)
            if len(moreDirs):
                dirsInDir.extend(moreDirs)

    return dirsInDir

# ------------------------------------------------------------------------------------------------------

def rename_all_jpg_in_all_sub_paths_to_exif_dates(rootPath, debug = False):
    if not os.path.isdir(rootPath):
        raise ValueError(f'Error! ->{rootPath}<- is not a directory!')

    subdirs = get_recursive_subdir_list(startPath=rootPath)

    for subdir in subdirs:
        print(f'{color.CBLUE2}EXIF change in process for {color.CWHITE}' + \
                f'{subdir} for {color.CYELLOW}{len(os.listdir(subdir))}' + \
                f'{color.CEND} files')
        rename_all_jpg_in_path_to_exif_dates(pathToJpgs = subdir)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_path_info(fullPathToFile = None):
    dirName, baseName, fileName, extension = get_path_info(fullPathToFile)

    print(f'Active file: {color.CYELLOW}{fullPathToFile}{color.CEND}')
    print(f'Dirname:     {color.CVIOLET}{dirName}{color.CEND}')
    print(f'Basename:    {color.CCYAN}{baseName}{color.CEND}')
    print(f'Filename:    {color.CWHITE}{fileName}{color.CEND}')
    print(f'Extension:   {color.CBLUE}{extension}{color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_exif_info(fullPathToFile = None):
    exifData = get_exif_data(fullPathToFile)

    if not exifData:
        print(f'{color.CRED2}Warning! {color.CWHITE2}No EXIF info {color.CRED2}' + \
                f'retrieved from specified file - {color.CWHITE2}' + \
                f'{fullPathToFile}{color.CEND}')
        return

    tagCount = 0

    for key, val in exifData.items():
        print(f'{color.CBLUE}{key}: {color.CEND}{val}')
        tagCount += 1

    '''
    for entry in exifData:
        entryStrList = entry.__str__().split(':')
        entryStrList = [ s.strip() for s in entryStrList ]

        print(f'{color.CBLUE}{entry.key()}: {color.CEND}{entryStrList[1:]}')
        tagCount += 1

        if tagCount == len(exifData):
            for attr in dir(entry):
                print(f' ---> {color.CGREEN}{attr}:{color.CEND} {getattr(entry, attr)}')
    '''

    '''
        if tagCount < len(exifData):
            print(f'{color.CYELLOW}{entry.familyName()}.{entry.groupName()}.{entry.tagName()}{color.CEND}: {entry.tagLabel()}')
            print(f'{color.CRED2}{entry.familyName()}.{entry.groupName()}.{entry.tagName()}{color.CEND}: {entry.print()}')
            print(f'{color.CGREEN}{entry.key()}{color.CEND} -- {entry.__str__()}')
            print(f'{color.CVIOLET2}{entry.__str__().split(":")[1]}{color.CEND}')
            print(f'{type(entry.value())}')
        else:
            print(f'{color.CYELLOW}{entry.familyName()}.{entry.groupName()}.{entry.tagName()}{color.CEND} dir: {dir(entry)}')
    '''

    '''
            import inspect
            for name, value in inspect.getmembers(entry):
                print(f' ---> {color.CVIOLET2}{name}:{color.CEND} {value}')
    '''

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




from media_mgr import (
        media_cfg,
        server_cfg,
        comms_utility,
        paths_and_drives,
        search,
        mover,
        gather,
        plex_upgrade,
        exif_rename,
        )



#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# Comms (and shell specific) utilities
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

import json
import sys
import subprocess

import asyncio, asyncssh

from quickcolor.color_def import color

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

# Note: this method of retrieving remote content is REALLY SLOW - zsh/bash kicks ass:
# -- show_drives_on_server 10.114.108.70 takes 250 msec
# -- running this python script takes
def run_ssh_send_cmd(ipv4: str, cmd: str = 'ls -als',
        timeout: int = 5, debug: bool = False):
    if debug:
        print(f'---- sending {color.CRED2}run_ssh_send_cmd{color.CEND} ' + \
                f'SEND CMD to {color.CBLUE2}{ipv4}:\n' + \
                f'{color.CBLUE}{cmd}{color.CEND}')

    session = subprocess.run(
            ["ssh", "%s" % ipv4, cmd],
            capture_output = True,
            text = True,
            timeout = timeout)

# ------------------------------------------------------------------------------------------------------

# Note: this method of retrieving remote content is REALLY SLOW - bash kicks ass:
# -- show_drives_on_server 10.114.108.70 takes 250 msec
# -- running this python script takes
def run_ssh_cmd(ipv4: str, cmd: str = 'ls -als',
        jsonOutput: bool = False, timeout: int = 5,
        debug: bool = False):
    if debug:
        print(f'---- sending {color.CYELLOW}run_ssh_cmd{color.CEND} ' + \
                f'CMD to {color.CBLUE2}{ipv4}:\n' + \
                f'{color.CYELLOW}{cmd}{color.CEND}')

    session = subprocess.run(
            ["ssh", "%s" % ipv4, cmd],
            stdout = subprocess.PIPE,
            stderr = subprocess.PIPE,
            # capture_output = True,
            text = True,
            timeout = timeout)

    try:
        if jsonOutput:
            returnResult = json.loads(session.stdout)
        else:
            result = session.stdout.strip("\n")
            returnResult = result.split('\n')

        if debug:
            print(f"{color.CRED}Received:{color.CEND}\n{str(returnResult)}")

        return returnResult

    except Exception as e:
        # error = session.stderr.readlines()
        error = session.stderr
        print(f'Error:\n{error}\n{str(e)}', file = sys.stderr)
        return None

# ------------------------------------------------------------------------------------------------------

def run_cmd(ipv4: str | None = None, cmd: str = 'ls -als',
            cmdGet: bool = True, jsonOutput: bool = False,
            timeout: int = 5, shell: bool = False,
            retCodeOnly: bool = False, debug: bool = False):

    if not ipv4:
        if debug:
            print(f'---- sending {color.CRED}run_cmd{color.CEND} ' + \
                    f'SEND CMD to {color.CBLUE2} the local system:\n' + \
                    f'{color.CBLUE}{cmd}{color.CEND}')

        cmdList = []
        cmdList.append(cmd)
        cmdResult = subprocess.run(
            cmdList,
            # stdout = subprocess.PIPE,
            # stderr = subprocess.PIPE,
            capture_output = True,
            text = True,
            shell = shell,
            timeout = timeout)

        if isinstance(cmdResult, subprocess.CompletedProcess) and cmdResult.returncode and debug:
            raise SystemError(f'Warning: Problem running local cmd ->{cmd}<- ' + \
                    f'... returncode: {cmdResult.returncode}\n... complete cmdREsult: {cmdResult}')

        if retCodeOnly:
            returnResult = cmdResult.returncode

        elif jsonOutput:
            returnResult = json.loads(cmdResult.stdout)
        else:
            result = cmdResult.stdout.strip("\n")
            returnResult = result.split('\n')

        if debug:
            print(f"{color.CRED}Received:{color.CEND}\n{str(returnResult)}")

        return returnResult

    else:
        if cmdGet:
            cmdResult = run_ssh_cmd(ipv4, cmd, jsonOutput, timeout, debug)
            if isinstance(cmdResult, subprocess.CompletedProcess) and cmdResult.returncode:
                raise SystemError(f'Warning: Problem running cmd ->{cmd}<- ' + \
                        f'on {ipv4} ... returncode: {cmdResult.returncode}')
        else:
            cmdResult = run_ssh_send_cmd(ipv4, cmd, timeout, debug)

    return cmdResult

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def is_server_active(ipv4: str = None):
    if not ipv4:
        raise ValueError(f"Need a valid IPV4 address!")

    retCode = run_cmd(ipv4 = None,
            cmd = f'ping -c 1 -W 1 -q {ipv4}',
            timeout = 2, shell = True, retCodeOnly = True)

    return retCode == 0

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def group_list(inputList = None, separator = None):
    group = []
    for item in inputList:
        if separator in item:
            yield group
            group = []
        group.append(item)
    yield group

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

async def run_ssh_cmd_async_task(ipv4: str, remoteCmd: str):
    async with asyncssh.connect(ipv4, known_hosts=None) as conn:
        try:
            # print(f'Running ->{remoteCmd}<- on {ipv4}')
            result = await conn.run(remoteCmd, check=True, timeout=None)
            # print(f'Result: {result.stdout}')
        except asyncssh.ProcessError as exc:
            print(exc.stderr, end='')
            print(f'ProcessError: Process exited with status {exc.exit_status}', file=sys.stderr)
            raise
        except asyncssh.ChannelOpenError as exc:
            print(exc.stderr, end='')
            print(f'ChannelOpenError: Process exited with status {exc.exit_status}', file=sys.stderr)
            raise
        except asyncssh.TimeoutError as exc:
            print(exc.stderr, end='')
            print(f'TimeoutError: Process exited with status {exc.exit_status}', file=sys.stderr)
            raise

        if result.returncode:
            raise ValueError(f"Dude, que passa?\n{result.stderr}")

        if '\n' in result.stdout:
            # print(f'result.stdout:\n->{result.stdout.split('\n')}<-')
            return (ipv4, [x for x in result.stdout.split('\n') if len(x)])

        # print(f'result.stdout:\n->{result.stdout}<-')
        return (ipv4, result.stdout)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

async def run_ssh_cmd_async_multi_server(ipv4List: list, remoteCmd: str):
    if not ipv4List:
        raise ValueError('Need a list of servers to reach out and touch!')

    tasks = []
    for ipv4 in ipv4List:
        tasks.append(asyncio.create_task(run_ssh_cmd_async_task(ipv4, remoteCmd)))
    return await asyncio.gather(*tasks)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def run_ssh_cmd_via_asyncio(ipv4List: list, remoteCmd: str):
    return asyncio.run(run_ssh_cmd_async_multi_server(ipv4List, remoteCmd))

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

async def run_ssh_cmds_async_multi_server(remoteCmdPerIpv4: dict):
    if not remoteCmdPerIpv4:
        raise ValueError('Need a dictionary of servers and cmds to reach out and touch!')

    tasks = []
    for ipv4, remoteCmd in remoteCmdPerIpv4.items():
        tasks.append(asyncio.create_task(run_ssh_cmd_async_task(ipv4, remoteCmd)))
    return await asyncio.gather(*tasks)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def run_ssh_cmds_via_asyncio(remoteCmdPerIpv4: dict):
    return asyncio.run(run_ssh_cmds_async_multi_server(remoteCmdPerIpv4))

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI for Comms Utility Methods
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from ..comms_utility import run_cmd, is_server_active

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Comms {color.CYELLOW2}Utilities {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument( '--ipv4', metavar='<address>',
                default=None, help='IPV4 to server')

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        subparsers = parser.add_subparsers(dest='cmd')

        p_dfh = subparsers.add_parser('dfh', help='List of drives (local or remote)')

        p_ls = subparsers.add_parser('ls', help='List files in dir')

        p_lsblk = subparsers.add_parser('lsblk', help='Show mountables')

        p_serverChk = subparsers.add_parser('server.chk', help='Server ping')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'dfh':
            # note quotes around grep argument
            # -- grep on zsh target env does not work without quotes
            cmd = 'df -h | grep "sd[b-z]" | sort -k 5'
            print(run_cmd(ipv4 = args.ipv4, cmd = cmd,
                cmdGet = True, debug = args.verbose))

        elif args.cmd == 'ls':
            print(run_cmd(ipv4 = args.ipv4, cmd = "ls -als .",
                cmdGet = True, debug = args.verbose))

        elif args.cmd == 'lsblk':
            print(run_cmd(ipv4 = args.ipv4, cmd = "lsblk -b --json",
                cmdGet = True, jsonOutput = True, debug = args.verbose))

        elif args.cmd == 'server.chk':
            serverInfo = f'-- Server {color.CYELLOW}{args.ipv4}{color.CEND}'
            if (is_server_active(ipv4 = args.ipv4)):
                print(f'{serverInfo} is {color.CGREEN}active{color.CEND}!')
            else:
                print(f'{serverInfo} is {color.CRED2}dead{color.CEND}!')

    except Exception as e:
        exception_details(e, "Comms Utility CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Media Gathering Routines
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

from collections import defaultdict

import json
import os
from datetime import datetime

from quickcolor.color_def import color
from quickcolor.color_filter import strip_ansi_esc_sequences_from_string
from delayviewer.stopwatch import Stopwatch, handle_stopwatch

from media_mgr.comms_utility import is_server_active

from media_mgr.media_cfg import MediaConfig
from media_mgr.server_cfg import ServerConfig

from media_mgr.paths_and_drives import get_filtered_media_paths
from media_mgr.search import get_num_titles, extract_search_path_collection

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_all_items_in_search_paths(ipv4 = None,
                                  serverType = 'plex',
                                  searchPathList = None,
                                  verbose: bool = False):
    if not searchPathList:
        searchPathList = get_filtered_media_paths(ipv4 = ipv4, serverType = serverType)

    cmd =''
    for path in searchPathList:
        # note - returned ls output always starts with a total <size> line -- the grep -wv removes it
        cmd += f'echo \"Drive.path: {path}\" ; ls --size -h {path} | grep -wv \"^total\" ; '

    # retrieve a dictionary list (titles by paths) unfiltered
    return extract_search_path_collection(ipv4 = ipv4,
                                          cmd = cmd,
                                          getAllGroups = True,
                                          verbose = verbose)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def sort_retrieved_items_by_type(collection, verbose: bool = False):
    sortedCollection = defaultdict(list)

    for path in collection:
        label = os.path.basename(path)
        driveName = os.path.basename(os.path.dirname(path))
        if collection[path]:
            modifiedCollection = []
            for entry in collection[path]:
                size, _, title = entry.lstrip().partition(' ')
                sortEntry = {}
                sortEntry['size'] = size
                sortEntry['drive'] = driveName
                sortEntry['title'] = title
                modifiedCollection.append(sortEntry)

            sortedCollection[label] += modifiedCollection

    if verbose:
        # print(json.dumps(sortedCollection, indent=4))
        pass

    for label in sortedCollection:
        sortedCollection[label] = sorted(sortedCollection[label], key=lambda x: x['title'])

    if verbose:
        # print(json.dumps(sortedCollection, indent=4))
        pass

    return sortedCollection

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_sorted_title_bundles(ipv4 = None, serverType = 'plex', verbose: bool = False):
    try:
        retrievedTitles = get_all_items_in_search_paths(ipv4 = ipv4,
                                                        serverType = serverType,
                                                        verbose = verbose)

    except Exception as e:
        print(f'{color.CRED2}-- Processing error: Sorted title bundle retrieval ' + \
                f'aborted for items on {color.CWHITE2}{ipv4}{color.CRED2} and ' + \
                f'server type {color.CWHITE}{serverType}\n' + \
                f'{color.CRED2}   Investigate {color.CWHITE}{ipv4}{color.CRED2}' + \
                f'for problems with drive mounts!{color.CEND}')
        return None

    if not retrievedTitles:
        location = str(ipv4) if ipv4 else 'local machine'
        print(f'\n{color.CRED2}-- Did not find any titles in any search path on {color.CYELLOW}{location}{color.CEND}')
        return None

    if verbose:
        # print(json.dumps(retrievedTitles, indent=4))
        pass

    return sort_retrieved_items_by_type(retrievedTitles, verbose = verbose)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_retrieved_titles(ipv4 = None, serverType = 'plex', verbose: bool = False):
    if not is_server_active(ipv4 = ipv4):
        print(f'\n{color.CWHITE2}-- Warning: {color.CRED2}Could not reach server ' + \
                f'{color.CYELLOW}{ipv4}{color.CRED2} -- it is dead!{color.CEND}')
        return

    retrievedTitles = []
    try:
        retrievedTitles = get_all_items_in_search_paths(ipv4 = ipv4,
                                                        serverType = serverType,
                                                        verbose = verbose)

        if verbose:
            # print(json.dumps(retrievedTitles, indent = 4))
            pass

    except Exception as e:
        print(f'{color.CRED2}-- Processing error: Title retrieval aborted for items on' + \
                f'{color.CWHITE}{ipv4}{color.CRED2} and server type {color.CWHITE}{serverType}\n' + \
                f'{color.CRED2}   Investigate {color.CWHITE}{ipv4}{color.CRED2} for problems ' + \
                f'with drive mounts!{color.CEND}')
        return

    if not retrievedTitles:
        location = str(ipv4) if ipv4 else "local machine"
        print(f"\n{colors.fg.red}-- Did not find any titles in any search path on {colors.fg.yellow}{location}{colors.off}")
        return

    mc = MediaConfig()

    location = str(ipv4) if ipv4 else "local machine"
    print(f'{color.CGREEN}-- Found {color.CWHITE}{get_num_titles(retrievedTitles)} ' + \
            f'{color.CGREEN}total titles on {color.CWHITE}{location}{color.CEND}')

    numTitles = 0
    for pathIdx, path in enumerate(retrievedTitles):
        _, colorCode = mc.get_color_label(os.path.basename(path))
        colorCode = colorCode if colorCode else color.CVIOLET

        if verbose:
            # print(json.dumps(retrievedTitles[path], indent=4))
            pass

        print(color.CRED2)
        print('- ' * 60)
        print(color.CEND)
        for titleIdx, entry in enumerate(retrievedTitles[path]):
            size, _, title = entry.lstrip().partition(' ')
            numTitles += 1
            print(f'{color.CVIOLET}{pathIdx:>3}-{titleIdx:<4} - {color.CWHITE}' + \
                    f'{numTitles:>3}. {colorCode}{path}/{title} {color.CWHITE2}({size}){color.CEND}')
        # input("Press enter to continue...")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_all_retrieved_titles():
    sc = ServerConfig()

    for name, serverCfg in sc.get_server_list():
        show_retrieved_titles(ipv4 = serverCfg['ipv4'], serverType = serverCfg['serverType'])

        print('-' * 120)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_header_in_collection(area, numTitlesInArea):
    header = f'\n{color.CRED2}'
    header += '- ' * 60

    if numTitlesInArea > 1:
        header += f'\n{color.CGREEN}-- There are {color.CWHITE}{numTitlesInArea} {color.CGREEN}titles in the'
    else:
        header += f'\n{color.CGREEN}-- There is {color.CWHITE}{numTitlesInArea} {color.CGREEN}title in the'

    return f'{header} {color.CYELLOW}{area} {color.CGREEN}area{color.CEND}\n'

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

@handle_stopwatch
def show_title_bundles(ipv4 = None, serverType = 'plex', stopwatch = None, verbose: bool = False):
    print()
    stopwatch.start(f'{color.CBLUE2}-- Bundle retrieval in progress ...{color.CEND}')
    sortedCollection = get_sorted_title_bundles(ipv4 = ipv4,
                                                serverType = serverType,
                                                verbose = verbose)
    if not sortedCollection:
        stopwatch.stop(f'{color.CRED2}Error in sorted collection retrieval!{color.CEND}')
        return
    stopwatch.stop()

    if verbose:
        # print(json.dumps(sortedCollection, indent = 4))
        pass

    mc = MediaConfig()

    for label in sortedCollection:
        print(get_header_in_collection(area = label,
                                       numTitlesInArea = len(sortedCollection[label])))

        _, colorCode = mc.get_color_label(label)
        colorCode = colorCode if colorCode else color.CRED

        numTitles = 0
        for entry in sortedCollection[label]:
            numTitles += 1
            print(f'{color.CWHITE}{numTitles:>3}. {colorCode}{entry["title"]} ' + \
                    f'{color.CWHITE2}({entry["size"]}){color.CEND} -- {entry["drive"]}')

    print('=' * 120)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

@handle_stopwatch
def store_title_bundles(ipv4 = None,
                        serverType = 'plex',
                        storePath = None,
                        verbose: bool = False,
                        stopwatch = None):
    print()
    stopwatch.start(f'{color.CBLUE2}-- Bundle retrieval in progress ...{color.CEND}')
    sortedCollection = get_sorted_title_bundles(ipv4 = ipv4,
                                                serverType = serverType,
                                                verbose = verbose)
    if not sortedCollection:
        stopwatch.stop(f'{color.CRED2}Error in sorted collection retrieval!{color.CEND}')
        return
    stopwatch.stop()

    print(f'{color.CBLUE2}-- Retrieved title bundles from {color.CWHITE}{ipv4}{color.CBLUE2} ', end='', flush=True)

    mc = MediaConfig()

    if not storePath:
        storePath = '/tmp'
    elif not os.path.isdir(storePath):
        storePath = '/tmp'

    fullBundlePath = f'{storePath}/title_bundles_{datetime.now().strftime("%Y-%m-%d_%H_%M_%S")}.txt'
    with open(fullBundlePath, "w") as fileHandle:

        for label in sortedCollection:
            fileHandle.write(strip_ansi_esc_sequences_from_string(get_header_in_collection(area = label,
                numTitlesInArea = len(sortedCollection[label]))) + '\n')

            for idx, entry in enumerate(sortedCollection[label]):
                fileHandle.write(f'{idx+1:>3}. {entry["title"]} -- {entry["size"]} -- {entry["drive"]}\n')

            fileHandle.write('\n')

    print(f'{color.CBLUE2}and storing to {color.CWHITE}{fullBundlePath}{color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

@handle_stopwatch
def store_title_bundles_plex_and_worker(ipv4Plex = None, ipv4Worker = None,
        storePath = None, stopwatch = None):

    print()
    stopwatch.start(f'{color.CBLUE2}-- Plex bundle retrieval in progress ...{color.CEND}')
    sortedCollection_Plex = get_sorted_title_bundles(ipv4 = ipv4Plex, serverType = 'plex')
    if not sortedCollection_Plex:
        stopwatch.stop(f'{color.CRED2}Error in retrieval!{color.CEND}')
        return
    stopwatch.stop()

    stopwatch.start(f'{color.CBLUE2}-- Worker bundle retrieval in progress ...{color.CEND}')
    sortedCollection_Worker = get_sorted_title_bundles(ipv4 = ipv4Worker, serverType = 'worker')
    if not sortedCollection_Worker:
        stopwatch.stop(f'{color.CRED2}Error in retrieval!{color.CEND}')
        return
    stopwatch.stop()

    print(f'{color.CBLUE2}-- Retrieved title bundles from {color.CWHITE}' + \
            f'{ipv4Plex}{color.CBLUE2} and {color.CWHITE}{ipv4Worker}' + \
            f'{color.CBLUE2} ', end='', flush=True)

    mc = MediaConfig()

    if not storePath:
        storePath = '/tmp'
    elif not os.path.isdir(storePath):
        storePath = '/tmp'

    sortedCollection = sortedCollection_Plex | sortedCollection_Worker

    fullBundlePath = f'{storePath}/title_bundles_{datetime.now().strftime("%Y-%m-%d_%H_%M_%S")}.txt'
    with open(fullBundlePath, "w") as fileHandle:
        for label in sortedCollection:
            fileHandle.write(strip_ansi_esc_sequences_from_string(get_header_in_collection(area = label,
                numTitlesInArea = len(sortedCollection[label]))) + "\n")

            for idx, entry in enumerate(sortedCollection[label]):
                fileHandle.write(f'{idx+1:>3}. {entry["title"]} -- {entry["size"]} -- {entry["drive"]}\n')

            fileHandle.write('\n')

    print(f'{color.CBLUE2}and storing to {color.CWHITE}{fullBundlePath}{color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI for Media Config
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.media_cfg import MediaConfig, mediaCategoryLkup, colorTypeLkup

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        preParsingMc = MediaConfig()

        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Media {color.CYELLOW2}Manager {color.CEND}configuration',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        subparsers = parser.add_subparsers(dest='cmd')

        p_regMediaCfg = subparsers.add_parser('reg', help='Register media label, category and color')
        p_regMediaCfg.add_argument('label', metavar='<label>', help='Media type label')
        p_regMediaCfg.add_argument('category', metavar='<category>',
                choices=mediaCategoryLkup.keys(), help='Media category')
        p_regMediaCfg.add_argument('dirColor', metavar='<dir-color>',
                choices=colorTypeLkup.keys(), help='Directory color')

        p_unregMediaCfg = subparsers.add_parser('unreg', help='Unregister specified media config')
        p_unregMediaCfg.add_argument('label', metavar='<existing.label>',
                choices=preParsingMc.get_configured_entries(), help='Media type label')

        p_showCfgLabels = subparsers.add_parser('show.labels', help='Show configured labels')

        p_getColorLabel = subparsers.add_parser('get.color.label', help='Get configured colorized label')
        p_getColorLabel.add_argument('label', metavar='<label>',
                choices=preParsingMc.get_configured_entries(), help='Media type label')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        mc = MediaConfig()

        if len(sys.argv) == 1:
            mc.show_media_config()
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'reg':
            mc.register_media_category(pathLabel = args.label,
                    category = args.category, dirColor = args.dirColor)
            mc.show_media_config()

        elif args.cmd == 'unreg':
            mc.unregister_media_category(pathLabel = args.label)
            mc.show_media_config()

        elif args.cmd == 'show.labels':
            mc.show_media_config_labels()

        elif args.cmd == 'get.color.label':
            colorLabel, colorCode = mc.get_color_label(pathLabel = args.label)
            print(f'\nThis label should display as ' + \
                    f'{colorCode}{colorLabel}{color.CEND} ...  ' + \
                    f'eg: {colorCode}{args.label}{color.CEND}')

    except Exception as e:
        exception_details(e, "Media Manager - Media Configuration CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Plex Media Server Upgrade Routines
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

import os
import re
import requests
import time

from urllib.parse import urlparse

from quickcolor.color_def import color
from delayviewer.spinner import Spinner, handle_spinner
from delayviewer.time_and_delay import time_show

from media_mgr.comms_utility import run_cmd, is_server_active
from media_mgr.media_cfg import MediaConfig
from media_mgr.server_cfg import ServerConfig

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def is_plex_server(ipv4 = None, debug = False) -> bool:
    if not ipv4:
        # local server check
        return os.path.isdir('/var/lib/plexmediaserver')

    cmd = f'[[ -d /var/lib/plexmediaserver ]] && echo -e "Plex Found"'
    items_received = run_cmd(ipv4, cmd, cmdGet = True, shell = True, debug = debug)
    return items_received[0] == 'Plex Found'

# ------------------------------------------------------------------------------------------------------

def get_running_plex_server_version(ipv4 = None, debug = False):
    if not is_plex_server(ipv4, debug):
        return 'Not a PLEX server!'

    cmd = f'dpkg --list | grep plexmediaserver | awk ' + "'{print $3}'"
    items_received = run_cmd(ipv4, cmd, cmdGet = True, shell=True, debug = debug)
    return items_received[0]

# ------------------------------------------------------------------------------------------------------

def extract_plex_upgrade_url_path():
    plexDownloadUrl = 'https://plex.tv/downloads/details/1?build=linux-ubuntu-x86_64&channel=16&distro=ubuntu'
    plexUpgradeInfo = requests.get(url = plexDownloadUrl, headers = {'Content-Type': 'application/json'})
    return re.search(r'(?<=url=\")(\S+)(?=\")', plexUpgradeInfo.text).group(0)

# ------------------------------------------------------------------------------------------------------

def get_available_plex_server_version():
    plexUpgradeUrlPath = extract_plex_upgrade_url_path()
    availableInstaller = os.path.basename(urlparse(plexUpgradeUrlPath).path)
    return re.search('plexmediaserver_(.*)_amd64.deb', availableInstaller, re.IGNORECASE).group(1)

# ------------------------------------------------------------------------------------------------------

def is_available_plex_server_version_same_as_running_version(ipv4 = None, debug = False):
    return get_available_plex_server_version() == get_running_plex_server_version(ipv4 = ipv4, debug = debug)

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def show_running_and_available_plex_server_versions(ipv4 = None, debug = False,
        spinner: Spinner = None):
    locationLabel = str(ipv4) if ipv4 else 'local_host'

    print()
    spinner.start(f'{color.CGREEN}-- Getting running and available for PLEX Server versions for ' + \
            f'{color.CWHITE2}{locationLabel}{color.CEND}', 0)
    runningPlexServerVer = get_running_plex_server_version(ipv4 = ipv4, debug = debug)
    availablePlexServerVer = get_available_plex_server_version()
    spinner.stop()

    spinner.start(f'{color.CBLUE}-- Analyzing PLEX Server versions for ' + \
            f'{color.CWHITE2}{locationLabel}{color.CBLUE} vs available online{color.CEND}', 0)
    if is_available_plex_server_version_same_as_running_version(ipv4 = ipv4, debug = debug):
        matchLabel = f'{color.CYELLOW}matching!{color.CEND}'
        runningColor = f'{color.CWHITE2}'
        availableColor = f'{color.CWHITE2}'
    else:
        matchLabel = f'{color.CRED}mismatching!{color.CEND}'
        runningColor = f'{color.CYELLOW2}'
        availableColor = f'{color.CYELLOW}'
    spinner.stop()

    print()
    print(f'{color.CBLUE2}  Running version: {runningColor}{runningPlexServerVer:<30}{matchLabel}')
    print(f'{color.CVIOLET}Available version: {availableColor}{availablePlexServerVer:<30}{matchLabel}')

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def get_available_plex_server_package_name(spinner:Spinner = None, verbose: bool = False):
    if verbose:
        spinner.start(f'-- Getting available {color.CRED2}PLEX installer{color.CEND} name', 0)
    plexUpgradeUrlPath = extract_plex_upgrade_url_path()
    availableInstaller = os.path.basename(urlparse(plexUpgradeUrlPath).path)

    if verbose:
        spinner.stop()
        print(f'{color.CCYAN}-- Available plex server binary: {color.CEND}{availableInstaller} ...')

    return availableInstaller

# ------------------------------------------------------------------------------------------------------

def does_installer_exist_on_target(ipv4 = None, pathToInstallerFile = None, debug = False):
    if ipv4:
        items_received = run_cmd(ipv4, f'ls {pathToInstallerFile}', cmdGet = True, debug = debug)
        return not all('' == s or s.isspace() for s in items_received)

    return os.path.exists(pathToInstallerFile)

# ------------------------------------------------------------------------------------------------------

def backup_existing_installer(ipv4 = None, pathToInstallerFile = None, debug = False):

    location = str(ipv4) if ipv4 else 'local_host'

    if does_installer_exist_on_target(ipv4 = ipv4,
            pathToInstallerFile = pathToInstallerFile, debug = debug):

        print(f'{color.CYELLOW}-- Installer {color.CWHITE2}{location}:' + \
                f'{pathToInstallerFile}{color.CYELLOW} exists and should be backed up {color.CEND}')

        if ipv4:
            renamed_file = f'{pathToInstallerFile}_remote_old'
            run_cmd(ipv4, f'mv {pathToInstallerFile} {renamed_file}', cmdGet=False, debug=debug)
        else:
            renamed_file = f'{pathToInstallerFile}_old'
            os.rename(pathToInstallerFile, f'{renamed_file}')

        print(f'   {color.CRED2}Note: Existing installer renamed: ' + \
                f'{color.CEND}{location}:{renamed_file}')

    else:
        print(f'{color.CBLUE}-- No trace of any existing installer ' + \
                f'{color.CWHITE2}{location}:{pathToInstallerFile}{color.CEND}')

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def retrieve_plex_server_bin(ipv4 = None, downloadDir = None,
        pathToInstallerFile = None, debug = False, spinner:Spinner = None):
    location = str(ipv4) if ipv4 else 'local_host'

    spinner.start(f'{color.CYELLOW}-- Extracting PLEX upgrade URL path{color.CEND}', 0)
    plexUpgradeUrlPath = extract_plex_upgrade_url_path()
    spinner.stop()

    spinner.start(f'{color.CGREEN}-- Downloading PLEX server binary ' + \
            f'{color.CWHITE2}{location}:{pathToInstallerFile}{color.CEND}', 0)
    wgetCmd = f'wget -P {downloadDir} {plexUpgradeUrlPath}'
    run_cmd(ipv4, wgetCmd, cmdGet = False, timeout = 60, shell = True, debug = debug)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def download_new_plex_server_archive(ipv4 = None, downloadDir = None, debug = False,
        spinner:Spinner = None):
    if not downloadDir:
        downloadDir='/tmp'

    if ipv4 and not is_server_active(ipv4 = ipv4):
        raise ConnectionError(f'Error: {ipv4} is not reachable!')

    location = str(ipv4) if ipv4 else 'local_host'

    spinner.start(f'{color.CVIOLET2}-- Deriving full path to incoming installer{color.CEND}', 0)
    fullPathToIncomingInstaller = downloadDir + '/' + get_available_plex_server_package_name()
    spinner.stop()

    backup_existing_installer(ipv4 = ipv4,
            pathToInstallerFile = fullPathToIncomingInstaller, debug = debug)

    retrieve_plex_server_bin(ipv4 = ipv4, downloadDir = downloadDir,
            pathToInstallerFile = fullPathToIncomingInstaller, debug = debug)

    return fullPathToIncomingInstaller

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

@handle_spinner
def install_plex_server(ipv4 = None, pathToInstallerFile = None,
        debug = False, spinner:Spinner = None):
    location = str(ipv4) if ipv4 else 'local_host'

    spinner.start(f'{color.CYELLOW}-- Installing new plex server to ' + \
            f'{color.CEND}{location}:{pathToInstallerFile}', 0)
    nextCmd = f'sudo dpkg -i {pathToInstallerFile}'
    run_cmd(ipv4, nextCmd, cmdGet = False, timeout = 120, shell = True, debug = debug)
    time.sleep(2)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def change_plex_user_and_group(ipv4 = None, user: str = 'edwaldner',
        group: str = 'edwaldner', debug = False, spinner:Spinner = None):

    spinner.start(f'{color.CBLUE}-- Changing plex user and group IDs to ' + \
            f'{color.CEND}{user} {color.CBLUE}and{color.CEND} {group}', 0)
    tmpCmd_1 = f'sudo sed -i \'s/User=plex/User={user}/\' /etc/systemd/system/multi-user.target.wants/plexmediaserver.service'
    tmpCmd_2 = f'sudo sed -i \'s/Group=plex/Group={group}/\' /etc/systemd/system/multi-user.target.wants/plexmediaserver.service'
    nextCmd = f'{tmpCmd_1}; {tmpCmd_2}'
    run_cmd(ipv4, nextCmd, cmdGet = False, timeout = 120, shell = True, debug = debug)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def change_plex_library_ownership(ipv4 = None, owner: str = 'edwaldner',
        debug = False, spinner:Spinner = None):

    spinner.start(f'{color.CGREEN}-- Changing ownership for {color.CEND}' + \
            f'/var/lib/plexmediaserver {color.CGREEN}to {color.CEND}{owner}', 0)
    # nextCmd = "date +\"%T.%3N\" | tee -a /tmp/chown_time ; sudo chown -R edwaldner. /var/lib/plexmediaserver; date +\"%T.%3N\" | tee -a /tmp/chown_time"
    nextCmd = 'sudo chown -R edwaldner. /var/lib/plexmediaserver'
    run_cmd(ipv4, nextCmd, cmdGet = False, timeout = 120, shell = True, debug = debug)
    time.sleep(1)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def reload_plex_daemon(ipv4 = None, debug = False, spinner:Spinner = None):
    spinner.start(f'{color.CVIOLET}-- Daemon reload{color.CEND}', 0)
    nextCmd = 'sudo systemctl --system daemon-reload'
    run_cmd(ipv4, nextCmd, cmdGet = False, timeout = 120, shell = True, debug = debug)
    time.sleep(1)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@handle_spinner
def restart_plex_service(ipv4 = None, debug = False, spinner:Spinner = None):
    spinner.start(f'{color.CBLUE2}-- Plex service restart{color.CEND}', 0)
    nextCmd = 'sudo systemctl restart plexmediaserver.service'
    run_cmd(ipv4, nextCmd, cmdGet = False, timeout = 120, shell = True, debug = debug)
    time.sleep(1)
    spinner.stop()

# ------------------------------------------------------------------------------------------------------

@time_show
def upgrade_plex_server(ipv4 = None, downloadDir = None, force = False,
        debug = False, spinner: Spinner = None):
    if not ipv4:
        import socket
        hostname = socket.gethostname()
        ipv4show = socket.gethostbyname(hostname)
    else:
        if ipv4 and not is_server_active(ipv4 = ipv4):
            raise ConnectionError(f'Error: {ipv4} is not reachable!')
        ipv4show = ipv4

    if not is_plex_server(ipv4, debug):
        print(f'{color.CRED2}Warning: {color.CWHITE2}{ipv4show}{color.CBLUE2} ' + \
                f'is not a PLEX server!{color.CEND}!\n')
        return

    if not force and is_available_plex_server_version_same_as_running_version(ipv4 = ipv4, debug = debug):
        print(f'{color.CRED2}Warning: {color.CWHITE2}{ipv4show}{color.CBLUE2} ' + \
                f'is running a plex server version matching the latest ' + \
                f'available version{color.CEND}!\n')

        runningPlexServerVer = get_running_plex_server_version(ipv4 = ipv4, debug = debug)
        availablePlexServerVer = get_available_plex_server_version()
        print(f'{color.CYELLOW}  Running version: {color.CWHITE2}{runningPlexServerVer}{color.CEND}')
        print(f'{color.CGREEN}Available version: {color.CWHITE2}{availablePlexServerVer}{color.CEND}')
        print(f'\n{color.CRED2}Skipping upgrade {color.CEND}...')
        return

    print()
    print(f'{color.CRED} Upgrading plex server {color.CWHITE2}{ipv4show}')
    print(f'{color.CYELLOW} -----------------------------------{color.CEND}')
    print(f'.. Currently running plex server version: {color.CBLUE2}' + \
            f'{get_running_plex_server_version(ipv4 = ipv4, debug = debug)}{color.CEND}')

    fullPathToInstallerFile = download_new_plex_server_archive(ipv4, downloadDir, debug)

    install_plex_server(ipv4 = ipv4,
            pathToInstallerFile = fullPathToInstallerFile, debug = debug)

    change_plex_user_and_group(ipv4 = ipv4, user = 'edwaldner',
            group = 'edwaldner', debug = debug)

    change_plex_library_ownership(ipv4 = ipv4, owner = 'edwaldner', debug = debug)

    reload_plex_daemon(ipv4 = ipv4, debug = debug)

    restart_plex_service(ipv4 = ipv4, debug = debug)

    # --------------------------------------------------------------------------------------------------

    if debug:
        nextCmd = 'sudo systemctl status plexmediaserver.service'
        print()
        for line in run_cmd(ipv4, nextCmd, cmdGet = True, debug = debug):
            print(line)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def upgrade_all_plex_servers(downloadDir = None, force = False, debug = False):
    sc = ServerConfig()

    for srvEntry in sc.get_server_list():
        srvName, srvCfg = srvEntry
        if srvCfg['serverType'] =='plex':
            upgrade_plex_server(srvCfg['ipv4'], downloadDir, force, debug)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI for Server Config
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.server_cfg import ServerConfig, serverTypeLkup

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        preParsingSc = ServerConfig()

        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Media {color.CYELLOW2}Server {color.CEND}configuration',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        subparsers = parser.add_subparsers(dest='cmd')

        p_regServerCfg = subparsers.add_parser('reg', help='Register server label, ipv4 and type')
        p_regServerCfg.add_argument('label', metavar='<label>', help='Media server label')
        p_regServerCfg.add_argument('ipv4', metavar='<ipv4-addr>', help='IPV4 address for the server')
        p_regServerCfg.add_argument('type', metavar='<server-type>',
                choices=serverTypeLkup.keys(), help='Media server type')

        p_unregServerCfg = subparsers.add_parser('unreg', help='Unregister specified media server')
        p_unregServerCfg.add_argument('label', metavar='<existing.label>',
                choices=preParsingSc.get_server_name_list(), help='Media server label')

        p_showCfgLabels = subparsers.add_parser('show.labels', help='Show configured labels')

        p_getSrvLabels = subparsers.add_parser('get.srv.labels', help='Debug - get configured server labels')

        p_getSrvInfo = subparsers.add_parser('get.srv.cfg', help='Debug - get server configs')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        sc = ServerConfig()

        if len(sys.argv) == 1:
            sc.show_server_config()
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'reg':
            sc.register_server(serverName = args.label,
                    ipv4 = args.ipv4, serverType = args.type)
            sc.show_server_config()

        elif args.cmd == 'unreg':
            sc.unregister_server(serverName = args.label)
            sc.show_server_config()

        elif args.cmd == 'show.labels':
            sc.show_server_config_labels()

        elif args.cmd == 'get.srv.labels':
            print(sc.get_server_name_list())

        elif args.cmd == 'get.srv.cfg':
            print(sc.get_server_list())

    except Exception as e:
        exception_details(e, "Media Manager - Server Configuration CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for Moving
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.media_cfg import MediaConfig

from media_mgr.mover import move_title, consolidate

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_move_plex():
    try:
        preMc = MediaConfig()

        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}Move {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action='store_true',
                            help='run with verbosity hooks enabled')

        parser.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        parser.add_argument('--test', action='store_true', help='Test move operation')

        parser.add_argument('from_basedir', metavar='<from.dir>',
                choices=preMc.get_configured_entries(), help='FROM base directory name')
        parser.add_argument('to_basedir', metavar='<to.dir>',
                choices=preMc.get_configured_entries(), help='TO base directory name')
        parser.add_argument("terms", metavar='<term_1> .. <term_n>',
                nargs=argparse.REMAINDER, help='search terms for move operation')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        move_title(ipv4 = args.ipv4, searchTerms = args.terms,
                fromBaseDir = args.from_basedir,
                toBaseDir = args.to_basedir,
                testMove = args.test,
                verbose = args.verbose)

    except Exception as e:
        exception_details(e, "Plex Media Relocation CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_consolidate():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}Media Consolidator {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        parser.add_argument('-v', '--verbose', action='store_true',
                            help='run with verbosity hooks enabled')

        parser.add_argument('--mediatype', default='mkv',
                            choices=['mkv', 'mp4', 'mov'], help='Media type (extensions)')
        parser.add_argument('--test', action='store_true', help='Test consolidate operation')

        parser.add_argument('ipv4', metavar='<ipv4.addr>', help='Server IPV4')
        parser.add_argument('rootAbsPath', metavar='<absolute.root.path>',
                help='top level directory from which to scan folders with media to consolidate')
        parser.add_argument('targetFolder', metavar='<tgt.folder.name>',
                help='Consolidation target folder name or \'.\'')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        consolidate(ipv4 = args.ipv4, rootAbsPath = args.rootAbsPath,
                    targetFolder = args.targetFolder,
                    mediaType = args.mediatype, testConsolidation = args.test,
                    verbose = args.verbose)

    except Exception as e:
        exception_details(e, "Plex Media Consolidation CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for Gathering
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.server_cfg import get_available_server_types

from media_mgr.gather import show_retrieved_titles, show_all_retrieved_titles
from media_mgr.gather import show_title_bundles, store_title_bundles
from media_mgr.gather import store_title_bundles_plex_and_worker

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}Gathering {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action='store_true',
                            help='run with verbosity hooks enabled')

        parser.add_argument('--version', action="store_true", help='top-level package version')

        sp = parser.add_subparsers(dest='cmd')

        sp_showRetrievedTitles = sp.add_parser('show.titles', help='Show retrieved titles')
        sp_showRetrievedTitles.add_argument('ipv4', metavar='<ipv4.addr>', help='Server IPV4')
        sp_showRetrievedTitles.add_argument('--type', choices=get_available_server_types(),
                default='plex', metavar='<server.type>', help='Server type')

        sp_showAllRetrievedTitles = sp.add_parser('show.all.titles', help='Show ALL retrieved titles')

        sp_showTitleBundles = sp.add_parser('show.bundles', help='Show title bundles')
        sp_showTitleBundles.add_argument('ipv4', metavar='<ipv4.addr>', help='Server IPV4')
        sp_showTitleBundles.add_argument('--type', choices=get_available_server_types(),
                default='plex', metavar='<server.type>', help='Server type')

        sp_storeTitleBundles = sp.add_parser('store.bundles', help='Store title bundles')
        sp_storeTitleBundles.add_argument('ipv4', metavar='<ipv4.addr>', help='Server IPV4')
        sp_storeTitleBundles.add_argument('--type', choices=get_available_server_types(),
                default='plex', metavar='<server.type>', help='Server type')
        sp_storeTitleBundles.add_argument('--path',
                default=None, metavar='<store.file.path>', help='Store file path')

        sp_showTitleBundlesPlexWorker = sp.add_parser('show.plex.n.worker.bundles',
                help='Show title bundles for Plex and Worker servers')
        sp_showTitleBundlesPlexWorker.add_argument('ipv4_plex',
                metavar='<ipv4.addr.plex.srvr>', help='Plex Server IPV4')
        sp_showTitleBundlesPlexWorker.add_argument('ipv4_worker',
                metavar='<ipv4.addr.worker.srvr>', help='Worker Server IPV4')

        sp_storeTitleBundlesPlexWorker = sp.add_parser('store.plex.n.worker.bundles',
                help='Store title bundles for Plex and Worker servers')
        sp_storeTitleBundlesPlexWorker.add_argument('ipv4_plex',
                metavar='<ipv4.addr.plex.srvr>', help='Plex Server IPV4')
        sp_storeTitleBundlesPlexWorker.add_argument('ipv4_worker',
                metavar='<ipv4.addr.worker.srvr>', help='Worker Server IPV4')
        sp_storeTitleBundlesPlexWorker.add_argument('--path',
                default=None, metavar='<store.file.path>', help='Store file path')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'show.titles':
            show_retrieved_titles(ipv4 = args.ipv4,
                                  serverType = args.type,
                                  verbose = args.verbose)

        elif args.cmd == 'show.all.titles':
            show_all_retrieved_titles()

        elif args.cmd == 'show.bundles':
            show_title_bundles(ipv4 = args.ipv4,
                               serverType = args.type,
                               verbose = args.verbose)

        elif args.cmd == 'store.bundles':
            store_title_bundles(ipv4 = args.ipv4,
                                serverType = args.type,
                                storePath = args.path,
                                verbose = args.verbose)

        elif args.cmd == 'show.plex.n.worker.bundles':
            show_title_bundles(ipv4 = args.ipv4_plex, serverType = 'plex')
            show_title_bundles(ipv4 = args.ipv4_worker, serverType = 'worker')

        elif args.cmd == 'store.plex.n.worker.bundles':
            store_title_bundles_plex_and_worker(ipv4Plex = args.ipv4_plex,
                    ipv4Worker = args.ipv4_worker, storePath = args.path)

    except Exception as e:
        exception_details(e, "Media Title Gathering CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Search Methods
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

from collections import defaultdict

import subprocess
import json
import os
import re

from delayviewer.time_and_delay import time_execution
from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.comms_utility import run_cmd, is_server_active, group_list

from media_mgr.media_cfg import MediaConfig
from media_mgr.server_cfg import ServerConfig

from media_mgr.paths_and_drives import get_filtered_media_paths

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# 2025-0201
# Search for titles in search path lists on servers
# - search is performed with run_cmd / subprocess via ssh
# - search contents are retrieved in lists for display
# - only search titles matching regex are returned
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

# @time_execution
def extract_search_path_collection(ipv4: str | None = None,
                                   cmd: str | None = None,
                                   getAllGroups: bool = False,
                                   shell: bool = False,
                                   verbose: bool = False):
    collection = defaultdict(list)

    cmdOutput = run_cmd(ipv4, cmd, shell = shell)
    if isinstance(cmdOutput, subprocess.CompletedProcess):
        if cmdOutput.returncode:
            raise ValueError(f'Warning: Problem retrieving command output!')

    if verbose:
        # print(json.dumps(cmdOutput,indent=4))
        pass

    medium = list(group_list(cmdOutput, 'Drive.path: '))
    if verbose:
        # print(json.dumps(medium,indent=4))
        pass

    for drivePathContents in medium:
        if not drivePathContents:
            continue
        groupId, groupContents = drivePathContents[:1], drivePathContents[1:]
        groupIdStr=''
        for groupIdElement in groupId:
            groupIdStr += groupIdElement

        if not groupIdStr:
            continue
        dumpIt, groupLabel = groupIdStr.split(' ')
        if groupContents or getAllGroups:
            collection[groupLabel] = groupContents

    return collection

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_matching_items_in_search_paths(ipv4: str | None = None,
                                       searchPathList: list | None = None,
                                       searchTerms : list | None = None,
                                       verbose: bool = False):
    searchRegex = ''
    for term in searchTerms:
        searchRegex += term + '.*'

    if verbose:
        '''
        print(json.dumps(searchPathList, indent = 4))
        print(f'{searchRegex=}')
        '''
        pass

    cmd = ''
    for path in searchPathList:
        cmd += f'echo \"Drive.path: {path}\" ; ls --size -h {path} | grep -i \'{searchRegex}\' ; '

    return extract_search_path_collection(ipv4 = ipv4, cmd = cmd, shell = True, verbose = verbose)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

# @time_execution
def find_items_in_search_paths(ipv4: str | None = None,
                               serverType = 'plex',
                               searchPathList: list | None = None,
                               searchTerms: list | None = None,
                               verbose: bool = False):
    if ipv4:
        if not is_server_active(ipv4 = ipv4):
            return defaultdict(list)

    if not searchPathList:
        searchPathList = get_filtered_media_paths(ipv4 = ipv4, serverType = serverType)

    if verbose:
        print(f'{color.CRED} -- Begin Search Path List -- {color.CEND}')
        print(json.dumps(searchPathList,indent=4))
        print(f'{color.CRED} -- End Search Path List -- {color.CEND}')
        pass

    collection = get_matching_items_in_search_paths(ipv4 = ipv4,
                                                    searchPathList = searchPathList,
                                                    searchTerms = searchTerms,
                                                    verbose = verbose)
    if verbose:
        '''
        print(f'{color.CRED} -- Begin Collection -- {color.CEND}')
        print(json.dumps(collection,indent=4))
        print(f'{color.CRED} -- End Collection -- {color.CEND}')
        '''
        pass

    # create a matched dictionary list (titles by paths)
    # filtering paths with matching titles (no empty paths)
    matchedTitles = defaultdict(list)
    for path in collection:
        if collection[path]:
            for item in collection[path]:
                size, _, title = item.lstrip().partition(' ')
                entry = {}
                entry['size'] = size
                entry['title'] = title
                matchedTitles[path].append(entry)

    if verbose:
        '''
        print(f'{color.CRED} -- Begin MatchedTitles -- {color.CEND}')
        print(json.dumps(matchedTitles,indent=4))
        print(f'{color.CRED} -- End MatchedTitles -- {color.CEND}')
        '''
        pass

    return matchedTitles

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_all_items_in_paths(ipv4: str | None = None,
                           pathList: list | None = None,
                           verbose: bool = False):
    cmd = ''
    for path in pathList:
        cmd += f'echo \"Drive.path: {path}\" ; ls --size -h {path} ; '

    collection = extract_search_path_collection(ipv4 = ipv4, cmd = cmd, shell = True, verbose = verbose)

    organizedTitles = defaultdict(list)
    for path in collection:
        if collection[path]:
            for item in collection[path]:
                size, _, title = item.lstrip().partition(' ')
                if size == 'total':
                    continue
                entry = {}
                entry['size'] = size
                entry['title'] = title
                organizedTitles[path].append(entry)

    if verbose:
        '''
        print(f'{color.CRED} -- Begin OrganizedTitles -- {color.CEND}')
        print(json.dumps(organizedTitles,indent=4))
        print(f'{color.CRED} -- End OrganizedTitles -- {color.CEND}')
        '''
        pass

    return organizedTitles

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

# @time_execution
def grab_all_titles_in_paths(ipv4: str | None = None,
                             serverType = 'plex',
                             pathList: list | None = None,
                             verbose: bool = False):

    if ipv4:
        if not is_server_active(ipv4 = ipv4):
            return defaultdict(list)

    filteredPathList = get_filtered_media_paths(ipv4 = ipv4, serverType = serverType, lookingFor = pathList)

    foundTitles = get_all_items_in_paths(ipv4 = ipv4, pathList = filteredPathList, verbose = verbose)

    if verbose:
        '''
        print(f'{color.CRED} -- Begin FoundTitles -- {color.CEND}')
        print(json.dumps(foundTitles,indent=4))
        print(f'{color.CRED} -- End FoundTitles -- {color.CEND}')
        '''
        pass

    return foundTitles

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
def get_num_titles(collection = None):
    numTitles = 0
    for path in collection:
        numTitles += len(collection[path])

    return numTitles

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_titles_in_paths(ipv4: str | None = None,
                        serverType = 'plex',
                        paths: list = [],
                        verbose: bool = False):

    if not len(paths):
        print(f'{color.CRED2}Warning{color.CEND}: No paths specified! Need paths... Need PATHS!!!!')
        return

    location = str(ipv4) if ipv4 else "local machine"

    try:
        allTitles = grab_all_titles_in_paths(ipv4 = ipv4,
                                             serverType = serverType,
                                             pathList = paths,
                                             verbose = verbose)
    except Exception as e:
        print(f'{color.CRED2}-- Processing error: Search aborted for all titles ' + \
                f'in paths on {color.CWHITE}{location}\n' + \
                f'{color.CRED}   Investigate {color.CWHITE}{location}{color.CRED2} ' + \
                f'for problems with drive mounts!{color.CEND}' + \
                f'\n{e}')
        exception_details(e, "organize_titles_in_paths run", raw=True)

        return

    print('')
    print('- ' * 50)

    if not allTitles:
        print(f'{color.CRED2}-- Did not find any titles on {color.CWHITE}{location}{color.CEND}')
        return

    mc = MediaConfig()
    print(f'{color.CGREEN}-- Found {color.CWHITE}{get_num_titles(allTitles)} ' + \
            f'{color.CGREEN}titles on {color.CWHITE}{location}{color.CEND}')

    if verbose:
        print(json.dumps(allTitles, indent = 4))
        return

    numProcessedTitles = 0
    for path in allTitles:
        _, colorCode = mc.get_color_label(os.path.basename(path))
        colorCode = colorCode if colorCode else color.CRED
        for entry in allTitles[path]:
            numProcessedTitles += 1
            print(f'{color.CWHITE}{numProcessedTitles:>3}. {colorCode}{path}/{entry["title"]}{color.CWHITE2} ' + \
                    f'({entry["size"]}){color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_matched_titles(ipv4: str | None = None,
                        serverType = 'plex',
                        searchTerms: list = ["the", "duff"],
                        verbose: bool = False):

    location = str(ipv4) if ipv4 else "local machine"

    try:
        matchedTitles = find_items_in_search_paths(ipv4 = ipv4,
                                                   serverType = serverType,
                                                   searchTerms = searchTerms,
                                                   verbose = verbose)

    except Exception as e:
        print(f'{color.CRED2}-- Processing error: Search aborted for titles ' + \
                f'matching {color.CWHITE}--> {color.CYELLOW}{" ".join(searchTerms)}' + \
                f'{color.CWHITE}<-- {color.CRED2}on {color.CWHITE}{location}\n' + \
                f'{color.CRED}   Investigate {color.CWHITE}{location}{color.CRED2} ' + \
                f'for problems with drive mounts!{color.CEND}' + \
                f'\n{e}')
        exception_details(e, "find_item_in_search_paths run", raw=True)

        return

    print('')
    print('- ' * 50)

    if not matchedTitles:
        print(f'{color.CRED2}-- Did not find any titles matching ' + \
                f'{color.CWHITE}--> {color.CYELLOW}{" ".join(searchTerms)} ' + \
                f'{color.CWHITE}<-- {color.CRED2}on {color.CWHITE}{location}{color.CEND}')
        return

    mc = MediaConfig()
    print(f'{color.CGREEN}-- Found {color.CWHITE}{get_num_titles(matchedTitles)} ' + \
            f'{color.CGREEN}titles matching {color.CWHITE}--> {color.CYELLOW}' + \
            f'{" ".join(searchTerms)} {color.CWHITE}<-- {color.CGREEN}on ' + \
            f'{color.CWHITE}{location}{color.CEND}')

    if verbose:
        # print(json.dumps(matchedTitles, indent = 4))
        return

    numMatchingTitles = 0
    for path in matchedTitles:
        _, colorCode = mc.get_color_label(os.path.basename(path))
        colorCode = colorCode if colorCode else color.CRED
        for entry in matchedTitles[path]:
            numMatchingTitles += 1
            print(f'{color.CWHITE}{numMatchingTitles:>3}. {colorCode}{path}/{entry["title"]}{color.CWHITE2} ' + \
                    f'({entry["size"]}){color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def show_all_matched_titles(searchTerms: list | None = None,
                            verbose: bool = False):
    srv = ServerConfig()

    for server in srv.get_server_name_list():
        show_matched_titles(ipv4 = srv.get_server_address(serverLabel = server),
                            serverType = srv.get_server_type(serverLabel = server),
                            searchTerms = searchTerms,
                            verbose = verbose)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Media Relocation Methods
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

from collections import defaultdict

import json
import os
import getpass
import paramiko

from quickcolor.color_def import color
from delayviewer.time_and_delay import time_execution

from media_mgr.comms_utility import run_cmd, is_server_active, group_list

from media_mgr.media_cfg import MediaConfig
from media_mgr.server_cfg import ServerConfig

from media_mgr.search import find_items_in_search_paths

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def move_title(ipv4: str | None = None, searchTerms: str | None = None,
        fromBaseDir: str = "Movies", toBaseDir: str = "Primo-Movies",
        testMove: bool = False, verbose: bool = False):

    if fromBaseDir == toBaseDir:
        print(f'\n{color.CRED2}-- Moving content from {color.CYELLOW2}{fromBaseDir} ' + \
                f'{color.CRED2}to {color.CYELLOW2}{toBaseDir} {color.CRED2}would ' + \
                f'be counter productive{color.CEND}!')
        return

    ipv4List = []
    if ipv4:
        ipv4List.append(ipv4)
    else:
        sc = ServerConfig()
        for srvName, srvCfg in sc.get_server_list():
            if srvCfg['serverType'] == 'plex':
                ipv4List.append(srvCfg['ipv4'])

    print("")
    for server in ipv4List:
        matchedTitles = []
        try:
            matchedTitles = find_items_in_search_paths(ipv4=server, searchPathList=None, searchTerms=searchTerms)

        except Exception as e:
            print(f'{color.CRED2}-- Processing error: Search aborted for ' + \
                    f'titles matching {color.CWHITE}--> {color.CYELLOW}' + \
                    f'{" ".join(searchTerms)} {color.CWHITE}<-- {color.CRED2}' + \
                    f'on {color.CWHITE}{server}\n{color.CRED2}   Investigate ' + \
                    f'{color.CWHITE}{server}{color.CRED2} for problems with ' + \
                    f'drive mounts!{color.CEND}\n{e}')
            return

        if not matchedTitles:
            test_move_msg = '(test move) ' if testMove else ''
            print(f'{color.CRED2}-- Did not find any titles matching ' + \
                    f'{color.CWHITE}--> {color.CYELLOW}{" ".join(searchTerms)}' + \
                    f'{color.CWHITE}<-- {color.CRED2}for {test_move_msg}' + \
                    f'relocation on {color.CWHITE}{server}{color.CEND}')
            continue

        if verbose:
            print(json.dumps(matchedTitles, indent=4))

        failedToMoveMatch = False
        numTitlesMoved = 0
        for key in matchedTitles:
            dirLabel = os.path.basename(key)
            dirRoot = os.path.dirname(key)

            for entry in matchedTitles[key]:
                title = entry['title']
                if dirLabel == fromBaseDir:
                    fromPathAndFile = key + "/" + title
                    toPath = dirRoot + "/" + toBaseDir
                    toPathAndFile = toPath + "/" + title

                    if not run_cmd(ipv4=server, cmd="ls \"%s\"" % (fromPathAndFile)):
                        print(f'-- {color.CRED}could not find {color.CWHITE2}' + \
                                f'{fromPathAndFile}{color.CRED2} on ' + \
                                f'{color.CYELLOW}{server}{color.CEND}')
                        failedToMoveMatch = True
                        continue

                    if not run_cmd(ipv4=server, cmd="[[ -d \"%s\" ]] && echo -e \"Found %s\"" % (toPath, toPath)):
                        print(f'-- {color.CRED}could not find path ' + \
                                f'{color.CWHITE2}{toPath}{color.CRED} on ' + \
                                f'{color.CYELLOW}{server}{color.CEND}')
                        failedToMoveMatch = True
                        continue

                    if testMove:
                        pretext = 'testing move of '
                    else:
                        pretext = 'moving '

                    print(f'-- {pretext}{color.CYELLOW}{title}{color.CEND} ' + \
                            f'residing on {color.CRED2}{server}{color.CEND}')
                    print(f'      from: {color.CBLUE2}{key}{color.CEND}')
                    print(f'        to: {color.CBLUE}{dirRoot}/{toBaseDir}{color.CEND}')
                    print('')

                    if not testMove:
                        cmd = "mv \"%s\" \"%s\"" % (fromPathAndFile, toPathAndFile)
                        if verbose:
                            print(f'... {color.CVIOLET2}running cmd:{color.CEND}\n{cmd}')
                        dumpThis = run_cmd(ipv4=server, cmd=cmd)
                        numTitlesMoved += 1

        if not testMove and numTitlesMoved == 0 and failedToMoveMatch == False:
            print(f'-- {color.CRED}could not find {color.CWHITE}any titles ' + \
                    f'{color.CRED}from path {color.CCYAN}{fromBaseDir} ' + \
                    f'{color.CRED} matching search terms ({color.CWHITE}' + \
                    f'{" ".join(searchTerms)}{color.CRED}) to move on ' + \
                    f'{color.CYELLOW}{server}{color.CEND}')

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

# 2025-0201 -- not currently used anywhere
def walk_local_directory(localPath: str, mediaType: str = 'mkv') -> dict:
    fileList = {}
    for (dirpath, dirnames, filenames) in os.walk(localPath):
        for filename in filenames:
            if filename.endswith(f'.{mediaType}'):
                fileList[filename] = os.sep.join([dirpath, filename])

    # print(json.dumps(fileList, indent=4))
    return fileList


def walk_remote_directory(sftp, remotePath: str):
    '''Recursively walk a directory on the remote server returning full file paths'''
    for entry in sftp.listdir_attr(remotePath):
        if entry.st_mode & 0o40000:  # Check if it's a directory
            yield from walk_remote_directory(sftp, f'{remotePath}/{entry.filename}')
        else:
            yield f'{remotePath}/{entry.filename}'


# @time_execution
def consolidate(ipv4: str, rootAbsPath: str, targetFolder: str,
                mediaType: str, testConsolidation: bool = False,
                verbose: bool = False):

    privatekeyfile = os.path.expanduser('~/.ssh/id_rsa')
    with paramiko.SSHClient() as ssh:
        '''
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(ipv4,
                     username = getpass.getuser(),
                     pkey = paramiko.RSAKey.from_private_key_file(privatekeyfile))
        '''
        ssh.load_system_host_keys()
        ssh.connect(ipv4)

        stdin, stdout, stderr = ssh.exec_command(f'[[ -d {rootAbsPath} ]] && echo -e \"Found path\"')
        if 'Found path' not in stdout.read().decode():
            raise FileNotFoundError(f'Error: {rootAbsPath} is not recognized as a valid directory!')

        if targetFolder != '.':
            consolidatedPath = f'{rootAbsPath}/{targetFolder}'

            stdin, stdout, stderr = ssh.exec_command(f'[[ -d {consolidatedPath} ]] && echo -e \"Found path\"')
            if 'Found path' in stdout.read().decode():
                raise ValueError(f'Error: {consolidatedPath} already exists! Remove or rename and retry...')

            stdin, stdout, stderr = ssh.exec_command(f'mkdir \"{consolidatedPath}\"')
            # stdin, stdout, stderr = ssh.exec_command(f'cd \"{consolidatedPath}\"; pwd; tree \"{consolidatedPath}\"')
            if verbose:
                print(stdout.read().decode())
        else:
            consolidatedPath = f'{rootAbsPath}'


        with ssh.open_sftp() as sftp:
            walkResults = walk_remote_directory(sftp, rootAbsPath)
            if verbose:
                print(f'{color.CYELLOW}Found by walking {color.CGREEN}{rootAbsPath}:{color.CEND}\n{walkResults}')
            numItemsConsolidated = 0
            for fileName in walkResults:
                if testConsolidation:
                    print(f'\n -- {color.CBLUE2}Test Moving {color.CWHITE2}{fileName}' + \
                            f'\n ........... {color.CBLUE2}to {color.CYELLOW}{consolidatedPath}{color.CEND}!')
                    numItemsConsolidated += 1
                    continue

                stdin, stdout, stderr = ssh.exec_command(f'mv \"{fileName}\" \"{consolidatedPath}\"')
                print(f'\n -- {color.CBLUE2}Moving {color.CWHITE2}{fileName}' + \
                        f'\n ...... {color.CBLUE2}to {color.CYELLOW}{consolidatedPath}{color.CEND}!')
                numItemsConsolidated += 1

            testPrefix = 'Test ' if testConsolidation else ''
            print(f'\n{testPrefix}Consolidated {color.CCYAN}{numItemsConsolidated}{color.CEND} items this run!\n')


# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for Paths & Drives
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.paths_and_drives import show_drive_info, show_drive_info_multiserver
from media_mgr.paths_and_drives import show_drive_paths, show_drive_paths_multiserver
from media_mgr.paths_and_drives import show_all_media_paths, show_all_media_paths_multiserver
from media_mgr.paths_and_drives import show_full_search_paths_all_drives, show_full_search_paths_all_drives_multiserver
from media_mgr.paths_and_drives import show_filtered_media_paths, get_filtered_media_paths, show_filtered_media_paths_multiserver
from media_mgr.paths_and_drives import get_lsblk_mountable_partitions
from media_mgr.paths_and_drives import show_dev_partitions, mount_dev_partitions

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Paths {color.CYELLOW2}and Drives {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        parser.add_argument('--version', action="store_true", help='top-level package version')

        subparsers = parser.add_subparsers(dest='cmd')

        p_showDriveInfo = subparsers.add_parser('drive.info', help='show drive info on a given server')
        p_showDriveInfo.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        p_asyncShowDriveInfo = subparsers.add_parser('async.drive.info', help='show drive info on list of servers')

        p_showDrivePaths = subparsers.add_parser('drive.paths', help='show drive paths on a server')
        p_showDrivePaths.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showDrivePaths.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')

        p_asyncShowDrivePaths = subparsers.add_parser('async.drive.paths', help='show drive paths on a list of servers')

        p_showAllMediaPaths = subparsers.add_parser('all.media.paths', help='show all media paths')
        p_showAllMediaPaths.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showAllMediaPaths.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')

        p_asyncShowAllMediaPaths = subparsers.add_parser('async.all.media.paths', help='show all media paths on a list of servers')

        p_showFullSearchPathsAllDrives = subparsers.add_parser('full.search.paths.all.drives',
                help='show full search paths from all drives')
        p_showFullSearchPathsAllDrives.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showFullSearchPathsAllDrives.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')

        p_asyncShowFullSearchPathsAllDrives = subparsers.add_parser('async.full.search.paths.all.drives',
                help='show full search paths from all drives on a list of servers')

        p_showFilteredMediaPaths = subparsers.add_parser('filtered.paths', help='show filtered local media paths')
        p_showFilteredMediaPaths.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showFilteredMediaPaths.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')
        p_showFilteredMediaPaths.add_argument('lookfor', default=None, nargs='*',
                metavar='<lookingFor>', help='Supported path terminations')

        p_asyncShowFilteredMediaPaths = subparsers.add_parser('async.filtered.paths', help='show filtered local media paths all servers')
        p_asyncShowFilteredMediaPaths.add_argument('lookfor', default=None, nargs='*',
                metavar='<lookingFor>', help='Supported path terminations')

        p_getLsblkDevicesAndMounts = subparsers.add_parser('get.lsblk.mounts', help='get lsblk devices and mount paths')
        p_getLsblkDevicesAndMounts.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        p_showDevicePartitions = subparsers.add_parser('show.dev.partitions', help='display device partitions (mounted and unmounted)')
        p_showDevicePartitions.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        p_mountDevicePartitions = subparsers.add_parser('mount.dev.partitions', help='mount any mountable device partitions')
        p_mountDevicePartitions.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)


        if args.cmd == 'drive.info':
            show_drive_info(ipv4 = args.ipv4)

        elif args.cmd == 'async.drive.info':
            show_drive_info_multiserver()

        elif args.cmd == 'drive.paths':
            show_drive_paths(ipv4 = args.ipv4, serverType = args.type)

        elif args.cmd == 'async.drive.paths':
            show_drive_paths_multiserver()

        elif args.cmd == 'all.media.paths':
            show_all_media_paths(ipv4 = args.ipv4, serverType = args.type)

        elif args.cmd == 'async.all.media.paths':
            show_all_media_paths_multiserver()

        elif args.cmd == 'full.search.paths.all.drives':
            show_full_search_paths_all_drives(ipv4 = args.ipv4, serverType = args.type)

        elif args.cmd == 'async.full.search.paths.all.drives':
            show_full_search_paths_all_drives_multiserver()

        elif args.cmd == 'filtered.paths':
            show_filtered_media_paths(ipv4 = args.ipv4,
                                      serverType = args.type,
                                      lookingFor = args.lookfor,
                                      paths = get_filtered_media_paths(ipv4 = args.ipv4,
                                                                       serverType = args.type,
                                                                       lookingFor = args.lookfor))

        elif args.cmd == 'async.filtered.paths':
            show_filtered_media_paths_multiserver(lookingFor = args.lookfor)

        elif args.cmd == 'get.lsblk.mounts':
            print('-' * 100)
            print(f'{color.CYELLOW}Get <lsblk> Devices & Mount Paths ({args.ipv4 if args.ipv4 else "Local"})!{color.CEND}')
            print('-' * 100)
            mountablePartitions = get_lsblk_mountable_partitions(ipv4=args.ipv4)
            for idx, partition in enumerate(mountablePartitions):
                print(f'{color.CBLUE}{idx+1:>3}. {color.CBLUE2}{json.dumps(partition, indent=4)}{color.CEND}')
                if idx % 20 == 19:
                    print('-' * 100)
            print('-' * 100)

        elif args.cmd == 'show.dev.partitions':
            show_dev_partitions(ipv4 = args.ipv4)

        elif args.cmd == 'mount.dev.partitions':
            mount_dev_partitions(ipv4 = args.ipv4)

    except Exception as e:
        exception_details(e, "Paths & Drives CLI", raw=True)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_mount_drives():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Mount Drives {color.CYELLOW2}on Server {color.CEND}utility',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        parser.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        mount_dev_partitions(ipv4 = args.ipv4)

    except Exception as e:
        exception_details(e, "Mount Drives CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for EXIF rename
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse
from argcomplete.completers import EnvironCompleter     # used for file/path arguments

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.exif_rename import show_path_info, show_exif_info, get_exif_datetime_original
from media_mgr.exif_rename import rename_all_jpg_in_path_to_exif_dates
from media_mgr.exif_rename import rename_all_jpg_in_all_sub_paths_to_exif_dates

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}EXIF Rename {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('--version', action="store_true", help='top-level package version')

        subparsers = parser.add_subparsers(dest='cmd')

        p_showPathInfo = subparsers.add_parser('path.info', help='Show path info')
        p_showPathInfo.add_argument("pathToFile", metavar='<path>',
                help='Full path to JPG file').completer = EnvironCompleter

        p_showExifInfo = subparsers.add_parser('exif.info', help='Show EXIF info')
        p_showExifInfo.add_argument("pathToFile", metavar='<path>',
                help='Full path to JPG file').completer = EnvironCompleter

        p_getExifDateTimeOrig = subparsers.add_parser('get.datetime', help='Get EXIF date/time original')
        p_getExifDateTimeOrig.add_argument("pathToFile", metavar='<path>',
                help='Full path to JPG file').completer = EnvironCompleter

        p_renameAllInPathToExif = subparsers.add_parser('to.exif.path', help='Rename all in path to EXIF')
        p_renameAllInPathToExif.add_argument("pathToFolder", metavar='<path>',
                help='Full path to JPG file folder').completer = EnvironCompleter

        p_renameAllInSubPathsToExif = subparsers.add_parser('all.to.exif',
                help='Rename all in path recursive to EXIF')
        p_renameAllInSubPathsToExif.add_argument("pathToRootFolder", metavar='<path>',
                help='Full path to JPG root folder').completer = EnvironCompleter

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'path.info':
            show_path_info(fullPathToFile = args.pathToFile)

        elif args.cmd == 'exif.info':
            show_exif_info(fullPathToFile = args.pathToFile)

        elif args.cmd == 'get.datetime':
            print(get_exif_datetime_original(fullPathToFile = args.pathToFile))

        elif args.cmd == 'to.exif.path':
            rename_all_jpg_in_path_to_exif_dates(pathToJpgs = args.pathToFolder)

        elif args.cmd == 'all.to.exif':
            rename_all_jpg_in_all_sub_paths_to_exif_dates(rootPath = args.pathToRootFolder)

    except Exception as e:
        exception_details(e, "EXIF File Rename CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Path and Drive Handling
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

from collections import defaultdict

import os
import getpass
import paramiko

import subprocess
import re
import json

from quickcolor.color_def import color
from delayviewer.spinner import Spinner, handle_spinner
from delayviewer.time_and_delay import time_show

from media_mgr.comms_utility import run_cmd, is_server_active
from media_mgr.comms_utility import run_ssh_cmd_via_asyncio
from media_mgr.comms_utility import run_ssh_cmds_via_asyncio
from media_mgr.media_cfg import MediaConfig
from media_mgr.server_cfg import ServerConfig

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_drive_stats(ipv4: str | None = None):
    # returns value is a list of strings (ie: CRLF separated lines)
    allDriveStats = run_cmd(ipv4, 'df -h | grep "sd[b-z]" | sort -k 6', shell = True)
    if allDriveStats == [''] or ('returncode' in allDriveStats and allDriveStats.returncode != 0):
        raise BlockingIOError(f'Warning: Could not retrieve drive paths from IPV4 {ipv4}!')

    return parse_drive_stats(allDriveStats)

# ------------------------------------------------------------------------------------------------------

def get_drive_stats_multiserver():
    sc = ServerConfig()
    ipv4List = sc.get_server_ipv4_list()

    allDriveStatsAllServers = run_ssh_cmd_via_asyncio(ipv4List, 'df -h | grep "sd[b-z]" | sort -k 6')
    parsedDriveStats = {}
    for allDriveStatsForServer in allDriveStatsAllServers:
        ipv4, allDriveStats = allDriveStatsForServer
        parsedDriveStats[ipv4] = parse_drive_stats(allDriveStats)

    return parsedDriveStats

# ------------------------------------------------------------------------------------------------------

def parse_drive_stats(allDriveStats: list):
    driveStats = defaultdict(list)

    for driveInfo in allDriveStats:
        if not len(driveInfo):
            continue
        # separate string into list of ascii fields separated by white space
        driveItems = re.sub(' +', ' ', driveInfo).split(' ')

        # allocate indexed item components into dictionary of lists
        # each list contains type specific contents
        driveStats['size'].append(driveItems[1])
        driveStats['used'].append(driveItems[2])
        driveStats['avail'].append(driveItems[3])
        driveStats['percent'].append(driveItems[4])
        driveStats['path'].append(driveItems[5])

    return driveStats

# ------------------------------------------------------------------------------------------------------

# @time_show
def show_drive_info(ipv4: str | None = None, driveStats: dict | None = None):
    if not driveStats:
        driveStats = get_drive_stats(ipv4 = ipv4)

    print(f'\n{color.CBLUE2}Drive Info     - {color.CBLUE}{ipv4 if ipv4 else "Local"}')
    print(f'{color.CWHITE}----- ----')

    for idx, zipItems in enumerate(zip(driveStats['size'], driveStats['used'],
        driveStats['avail'], driveStats['percent'], driveStats['path'])):
        size, used, avail, percent, path = zipItems
        print(f'{color.CGREEN}{idx+1:>3}. {path:<40} {color.CYELLOW} ' + \
                f'{size:<5} {used:<5} {avail:<5} {color.CCYAN}' + \
                f'{percent:>5}{color.CEND}')

# ------------------------------------------------------------------------------------------------------

@time_show
def show_drive_info_multiserver():
    multiServerDriveStats = get_drive_stats_multiserver()

    for ipv4, driveStats in multiServerDriveStats.items():
        show_drive_info(ipv4 = ipv4, driveStats = driveStats)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_drive_paths(ipv4: str | None = None, serverType: str = 'plex'):
    if serverType != 'plex':
        return [ run_cmd(ipv4, "pwd")[0] + "/Desktop/Loris-Lots" ]

    cmdResult = run_cmd(ipv4, "df -h | grep 'sd[b-z]' | sort -k 6 | awk '{ print $6 }'", shell = True)
    if type(cmdResult) is subprocess.CompletedProcess and cmdResult.returncode != 0:
        raise BlockingIOError(f"Warning: Could not retrieve drive paths from IPV4 {ipv4}!")

    return cmdResult

# ------------------------------------------------------------------------------------------------------

def get_drive_paths_multiserver():
    sc = ServerConfig()

    ipv4List = []
    for srvLabel, srvInfo in sc.get_server_list():
        if srvInfo['serverType'] == 'plex':
            ipv4List.append(srvInfo['ipv4'])

    return run_ssh_cmd_via_asyncio(ipv4List, "df -h | grep 'sd[b-z]' | sort -k 6 | awk '{ print $6 }'")

# ------------------------------------------------------------------------------------------------------

@time_show
def show_drive_paths(ipv4: str | None = None, serverType: str = 'plex'):
    print('-' * 100)
    print(f'{color.CYELLOW}Drive Paths ({ipv4 if ipv4 else "Local"})!{color.CEND}')
    print('-' * 100)
    paths = get_drive_paths(ipv4 = ipv4, serverType = serverType)
    for idx, path in enumerate(paths):
        print(f'{color.CGREEN2}{idx+1:>3}. {color.CEND}{color.CGREEN}{path}{color.CEND}')
        if idx % 20 == 19:
            print('-' * 100)
    print('-' * 100)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_drive_paths_multiserver():
    for ipv4, pathList in get_drive_paths_multiserver():
        print('-' * 100)
        print(f'{color.CYELLOW}Drive Paths ({ipv4 if ipv4 else "Local"})!{color.CEND}')
        print('-' * 100)
        for idx, path in enumerate(pathList):
            print(f'{color.CGREEN2}{idx+1:>3}. {color.CEND}{color.CGREEN}{path}{color.CEND}')
            if idx % 20 == 19:
                print('-' * 100)
        # print('-' * 100)
        print()

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_all_media_paths(ipv4: str | None = None, serverType: str = 'plex'):
    '''
    For Plex servers, retrieve all existing drive mounts with media sub-paths that actually exist
    in the file system
    '''
    paths = run_cmd(ipv4, f'ls -d /media/{getpass.getuser()}/*/*', shell = True) if serverType == 'plex' else \
            run_cmd(ipv4, f'ls -d /home/{getpass.getuser()}/Desktop/Loris-Lots/* | grep Incoming', shell = True)

    if type(paths) is subprocess.CompletedProcess:
        raise BlockingIOError(f"Warning: Could not retrieve drive paths from IPV4 {ipv4}!")

    return sorted(paths)

# ------------------------------------------------------------------------------------------------------

def get_all_media_paths_multiserver():
    '''
    For Plex servers, retrieve all existing drive mounts with media sub-paths that actually exist
    in the file system for each of the specified servers (based on discrete server types)
    '''
    sc = ServerConfig()

    remoteCmdPerIpv4 = {}
    for srvLabel, srvInfo in sc.get_server_list():
        if srvInfo['serverType'] == 'plex':
            remoteCmdPerIpv4[srvInfo['ipv4']] = f'ls -d /media/{getpass.getuser()}/*/*'
        else:
            remoteCmdPerIpv4[srvInfo['ipv4']] = f'ls -d /home/{getpass.getuser()}/Desktop/Loris-Lots/* | grep Incoming'

    return run_ssh_cmds_via_asyncio(remoteCmdPerIpv4)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_all_media_paths(ipv4: str | None = None, serverType: str = 'plex', paths: list | None = None):
    if not paths:
        paths = get_all_media_paths(ipv4 = ipv4, serverType = serverType)

    print('-' * 100)
    print(f'{color.CYELLOW}Get All Media Paths ({ipv4 if ipv4 else "Local"})!{color.CEND}')
    print('-' * 100)
    lastDriveName = 'unknown'
    idxColor = color.CGREEN2
    driveColor = color.CGREEN
    for idx, path in enumerate(paths):
        driveName = path.split('/')[3]
        if driveName != lastDriveName:
            lastDriveName = driveName
            idxColor = color.CYELLOW2 if idxColor == color.CGREEN2 else color.CGREEN2
            driveColor = color.CYELLOW if driveColor == color.CGREEN else color.CGREEN
            print('-' * 100)
        print(f'{idxColor}{idx+1:>3}. {driveColor}{path}{color.CEND}')
    print('-' * 100)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_all_media_paths_multiserver():
    for ipv4, allMediaPaths in get_all_media_paths_multiserver():
        show_all_media_paths(ipv4 = ipv4, paths = allMediaPaths)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def create_full_search_path_list_for_drive(drivePath: str):
    mc = MediaConfig()
    searchNameList = mc.get_configured_entries()

    fullSearchPathListForDrive = []
    for searchName in searchNameList:
        fullSearchPathListForDrive.append(f'{drivePath}/{searchName}')

    return fullSearchPathListForDrive

# ------------------------------------------------------------------------------------------------------

def get_full_search_paths_all_drives(ipv4: str | None = None, serverType: str = 'plex'):
    '''
    For Plex servers, merge all discovered drive mount paths with possible media sub-paths
    Media sub-paths are specified in pathNames.json in the personal cfg dir
    These are potential paths configured, not necessarily paths that exist
    The filtering method below is intended to filter out what does not exist
    as well as specific paths that exist but that are not part of config!
    '''
    drivePaths = get_drive_paths(ipv4, serverType)

    fullSearchPathsAllDrives = []

    if serverType == 'plex':
        for drivePath in drivePaths:
            searchPathListForDrive = create_full_search_path_list_for_drive(drivePath)
            fullSearchPathsAllDrives += searchPathListForDrive

    elif serverType == 'worker':
        fullSearchPathsAllDrives = get_all_media_paths(ipv4, serverType)

    return sorted(fullSearchPathsAllDrives)

# ------------------------------------------------------------------------------------------------------

def get_full_search_paths_all_drives_multiserver():
    sc = ServerConfig()

    remoteCmdPerIpv4 = {}
    for srvLabel, srvInfo in sc.get_server_list():
        if srvInfo['serverType'] == 'plex':
            remoteCmdPerIpv4[srvInfo['ipv4']] = f'ls -d /media/{getpass.getuser()}/*/*'
        else:
            remoteCmdPerIpv4[srvInfo['ipv4']] = f'ls -d /home/{getpass.getuser()}/Desktop/Loris-Lots/* | grep Incoming'

    return run_ssh_cmds_via_asyncio(remoteCmdPerIpv4)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_full_search_paths_all_drives(ipv4: str | None = None, serverType: str = 'plex', paths: list | None = None):
    if not paths:
        paths = get_full_search_paths_all_drives(ipv4 = ipv4, serverType = serverType)

    print('-' * 100)
    print(f'{color.CYELLOW}Get Full Search Paths All Drives from {ipv4 if ipv4 else "Local"}!{color.CEND}')
    print('-' * 100)
    lastDriveName = 'unknown'
    idxColor = color.CBLUE
    pathColor = color.CCYAN
    for idx, path in enumerate(paths):
        driveName = path.split('/')[3]
        if driveName != lastDriveName:
            lastDriveName = driveName
            idxColor = color.CYELLOW if idxColor == color.CBLUE else color.CBLUE
            pathColor = color.CYELLOW if pathColor == color.CCYAN else color.CCYAN
            print('-' * 100)
        print(f'{idxColor}{idx+1:>3}. {pathColor}{path}{color.CEND}')
    print('-' * 100)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_full_search_paths_all_drives_multiserver(ipv4: str | None = None, serverType: str = 'plex'):
    for ipv4, paths in get_full_search_paths_all_drives_multiserver():
        show_full_search_paths_all_drives(ipv4 = ipv4, paths = paths)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_filtered_media_paths(ipv4: str | None = None, serverType: str = 'plex', lookingFor: list | None = None):
    '''
    Taking all media paths that exist (drive mount paths plus media sub paths) and filter against
    all possible search paths on all mount paths - subset represents searchable paths that exist
    in the file system and that are part of the configured pathNames.json group
    '''
    paths = get_all_media_paths(ipv4, serverType)

    if lookingFor:
        paths = [ path for path in paths if os.path.basename(path) in lookingFor ]
    fullPaths = get_full_search_paths_all_drives(ipv4, serverType)

    if len(paths):
        return sorted(list(set(paths).intersection(fullPaths)))

    return []

# ------------------------------------------------------------------------------------------------------

def get_filtered_media_paths_multiserver(lookingFor: list | None = None):
    '''
    Taking all media paths that exist on multiple servers (drive mount paths plus media sub paths)
    and filter against all possible search paths on all mount paths - subset represents searchable paths
    that exist in the file system on all servers and that are part of the configured pathNames.json group
    '''
    allPaths = get_all_media_paths_multiserver()
    fullPaths = get_full_search_paths_all_drives_multiserver()
    referencePaths = {}
    for ipv4, paths in fullPaths:
        referencePaths[ipv4] = paths

    filteredPaths = []
    for ipv4, paths in allPaths:
        if lookingFor:
            paths = [ path for path in paths if os.path.basename(path) in lookingFor ]

        print(f'Paths for {ipv4} while looking for {lookingFor} -- len paths = {len(paths)}\n{paths}')
        if len(paths):
            filteredPaths.append((ipv4, sorted(list(set(paths).intersection(referencePaths[ipv4])))))
        else:
            filteredPaths.append((ipv4, []))

    return filteredPaths

# ------------------------------------------------------------------------------------------------------

@time_show
def show_filtered_media_paths(ipv4: str | None = None, serverType: str = 'plex',
                              lookingFor: list | None = None, paths: list | None = None):
    print('-' * 100)
    print(f'{color.CVIOLET2}Get Filtered Media Paths ({ipv4 if ipv4 else "Local"})!{color.CEND}')
    print('-' * 100)
    lastDriveName = 'unknown'
    idxColor = color.CBLUE
    pathColor = color.CCYAN
    for idx, path in enumerate(paths):
        driveName = path.split('/')[3]
        if driveName != lastDriveName:
            lastDriveName = driveName
            idxColor = color.CYELLOW if idxColor == color.CBLUE else color.CBLUE
            pathColor = color.CYELLOW if pathColor == color.CCYAN else color.CCYAN
            print('-' * 100)
        print(f'{idxColor}{idx+1:>3}. {pathColor}{path}{color.CEND}')
    print('-' * 100)

# ------------------------------------------------------------------------------------------------------

@time_show
def show_filtered_media_paths_multiserver(lookingFor: list | None = None):
    for ipv4, paths in get_filtered_media_paths_multiserver(lookingFor = lookingFor):
        show_filtered_media_paths(ipv4 = ipv4, paths = paths)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def get_lsblk_devices_and_mounts(ipv4: str | None = None):
    cmdResult = run_cmd(ipv4, "lsblk -b --json", jsonOutput=True)
    # cmdResult = run_cmd(ipv4, "lsblk -b | grep 'sd[b-z]' | grep 'part|disk' | awk '{ print $1, $4, $7 }'")
    if type(cmdResult) is subprocess.CompletedProcess:
        raise BlockingIOError(f"Warning: Could not retrieve devices and mount paths from IPV4 {ipv4}!")

    # print(json.dumps(cmdResult, indent=4))
    mountableDevices = []
    for dev in cmdResult['blockdevices']:
        if re.match("sd[b-z]", dev['name']):
            mountableDevices.append(dev)

    return mountableDevices

# ------------------------------------------------------------------------------------------------------

def get_lsblk_mountable_partitions(ipv4: str | None = None):
    mountableDevices = get_lsblk_devices_and_mounts(ipv4=ipv4)
    mountablePartitions = []
    for dev in mountableDevices:
        for child in dev['children']:
            if child['size'] > 1_000_000_000:
                mountablePartitions.append(child)

    return mountablePartitions

# ------------------------------------------------------------------------------------------------------

def show_dev_partitions(ipv4: str | None = None,
        partitionsThatAreAlreadyMounted: list | None = None,
        partitionsThatAreUnmounted : list | None = None):
    availablePartitions = []
    if not partitionsThatAreAlreadyMounted:
        availablePartitions = get_lsblk_mountable_partitions(ipv4=ipv4)
        partitionsThatAreAlreadyMounted = [ x for x in availablePartitions if x['mountpoints'] != [ None ] ]

    if not partitionsThatAreUnmounted:
        if not availablePartitions:
            availablePartitions = get_lsblk_mountable_partitions(ipv4=ipv4)
        partitionsThatAreUnmounted = [ x for x in availablePartitions if x['mountpoints'] == [ None ] ]

    if partitionsThatAreAlreadyMounted:
        print(f'\n{color.CBLUE2}-- The following {color.CWHITE}{len(partitionsThatAreAlreadyMounted)}' + \
                f'{color.CBLUE2} devices on {color.CWHITE}{ipv4}{color.CBLUE2} are mounted!{color.CEND}\n')
        for idx, partition in enumerate(partitionsThatAreAlreadyMounted):
            print(f'   {color.CGREEN2}{idx+1:>3}. {color.CEND}Device ' + \
                    f'{color.CCYAN}/dev/{partition["name"]}{color.CEND} ' + \
                    f'mount point is {color.CYELLOW}{partition["mountpoints"][0]}{color.CEND}')

    if partitionsThatAreUnmounted:
        print(f'\n{color.CBLUE2}-- The following {color.CWHITE}{len(partitionsThatAreUnmounted)}' + \
                f'{color.CBLUE2} devices on {color.CWHITE}{ipv4}{color.CBLUE2} are not mounted!{color.CEND}\n')
        for idx, partition in enumerate(partitionsThatAreUnmounted):
            print(f'   {color.CGREEN2}{idx+1:>3}. {color.CEND}Device ' + \
                    f'{color.CCYAN}/dev/{partition["name"]}{color.CEND} has no mount point!')

# ------------------------------------------------------------------------------------------------------

@time_show
@handle_spinner
def mount_dev_partitions(ipv4: str | None = None, spinner: Spinner | None = None):
    if ipv4 and not is_server_active(ipv4 = ipv4):
        raise ConnectionError(f'Error: {ipv4} is not reachable!')

    availablePartitions = get_lsblk_mountable_partitions(ipv4 = ipv4)
    partitionsThatAreUnmounted = [ x for x in availablePartitions if x['mountpoints'] == [ None ] ]
    partitionsThatAreAlreadyMounted = [ x for x in availablePartitions if x['mountpoints'] != [ None ] ]
    print('')
    if not partitionsThatAreUnmounted:
        print(f'{color.CRED2}-- There are no available partitions to mount! ' + \
                f'{color.CWHITE}All device partitions are mounted!{color.CEND}')
        show_dev_partitions(ipv4 = ipv4,
                partitionsThatAreAlreadyMounted = partitionsThatAreAlreadyMounted,
                partitionsThatAreUnmounted = partitionsThatAreUnmounted)
        return

    print(f'....  {color.CYELLOW2}{len(partitionsThatAreUnmounted)}' + \
            f'{color.CEND} of {color.CVIOLET2}{len(availablePartitions)}' + \
            f'{color.CEND} partitions are available to be mounted!')

    for idx, partition in enumerate(partitionsThatAreUnmounted):
        '''
        mountingPhrase = f'       Mounting {color.CCYAN}/dev/{partition["name"]}{color.CEND} ......   {color.CYELLOW}'
        print(mountingPhrase, end=' ', flush=True)
        '''
        spinner.start(f'{color.CGREEN}-- {color.CWHITE}{idx+1:>3}. ' + \
                f'{color.CGREEN}Mounting {color.CCYAN}/dev/{partition["name"]}' + \
                f'{color.CEND} ...', 60)
        cmdResult = run_cmd(ipv4, f"udisksctl mount -b /dev/{partition['name']}", timeout=15)
        spinner.stop()
        '''
        print('\r', end='', flush=True)
        print(f'{color.CGREEN}-- {color.CWHITE}{idx+1:>3}. ' + \
                f'{color.CGREEN}Mounted {color.CCYAN}/dev/{partition["name"]}' + \
                f'{color.CEND} in {color.CVIOLET2}{duration}{color.CEND}')
        '''

    show_dev_partitions(ipv4 = ipv4)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- Media Config
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

from quickcolor.color_def import color

from corecfg.multi_entity import MultiEntityCoreCfg

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

class MediaCategory():
    movie = f'{color.CYELLOW2}movie{color.CEND}'
    show = f'{color.CBLUE2}show{color.CEND}'
    special = f'{color.CCYAN2}specials{color.CEND}'
    standup = f'{color.CRED2}standup{color.CEND}'
    sport = f'{color.CGREEN}sports{color.CEND}'
    concert = f'{color.CVIOLET2}concert{color.CEND}'

mediaCategoryLkup = {
        'movie' : MediaCategory.movie,
        'show' : MediaCategory.show,
        'special' : MediaCategory.special,
        'standup' : MediaCategory.standup,
        'sport' : MediaCategory.sport,
        'concert' : MediaCategory.concert,
        }

colorTypeLkup = {
        'red' : color.CRED,
        'bold red' : color.CRED2,
        'yellow' : color.CRED,
        'bold yellow' : color.CYELLOW2,
        'blue' : color.CBLUE,
        'bold blue' : color.CBLUE2,
        'green' : color.CGREEN,
        'bold green' : color.CGREEN2,
        'cyan' : color.CCYAN,
        'bold cyan' : color.CCYAN2,
        'violet' : color.CVIOLET,
        'bold violet' : color.CVIOLET2,
        'white' : color.CWHITE,
        'bold white' : color.CWHITE2,
        }
# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

class MediaConfig(MultiEntityCoreCfg):

    # --------------------------------------------------------------------------------------------------
    def __init__(self, verbose: bool = False, debug: bool = False):
        self._verbose = verbose
        self._debug = debug
        if debug:
            print(f'{color.CCYAN2}Initializing {type(self).__name__}! {color.CWHITE2}id: {str(id(self))}{color.CEND}')

        super(MediaConfig, self).__init__(cfgFileName = 'media_config.json', debug = debug)

        cfgMetadataInit = { 'label' : 'Media Cfg', 'maxLabelLen' : 30 }
        categoryMetadataInit = {
                'category' : { 'color' : color.CBLUE2, 'maxLen' : 20 },
                'color' : { 'color' : color.CWHITE, 'maxLen' : 20 },
                }
        if cfgMetadataInit != self.get_cfg_metadata() or categoryMetadataInit != self.get_category_metadata():
            if debug:
                print(f'Initializing {type(self).__name__} metadata for cfg and categories!')
            self.initialize_metadata(cfgMetadata=cfgMetadataInit, categoryMetadata=categoryMetadataInit)

    # --------------------------------------------------------------------------------------------------
    def register_media_category(self, pathLabel: str, category: str, dirColor: str):
        if category not in mediaCategoryLkup.keys():
            raise ValueError(f"Media category ({category}) is not recognized! Try one of these: {mediaCategoryLkup.keys()}")

        if dirColor not in colorTypeLkup.keys():
            raise ValueError(f"Path color ({dirColor}) is not recognized! Try one of these: {colorTypeLkup.keys()}")

        tupleList = {}
        tupleList['category'] = category
        tupleList['color'] = dirColor

        '''
        allItems = self.get_all_configured_items()
        if tupleList in allItems.values():
            print(f'{color.CRED2}Error: {color.CWHITE}Tuple {color.CBLUE2}' + \
                    f'{str(tupleList.values())}{color.CWHITE} is already configured!{color.CEND}')
            return
        '''

        self.update_item(itemLabel = pathLabel, newTuple = tupleList)

    # --------------------------------------------------------------------------------------------------
    def unregister_media_category(self, pathLabel: str):
        allItems = self.get_all_configured_items()
        if pathLabel not in allItems.keys():
            print(f'{color.CRED2}Error: {color.CWHITE}Media cfg label ' + \
                    f'{color.CBLUE2}{pathLabel}{color.CWHITE}' + \
                    f' is not registered in media config!{color.CEND}')
            return

        print(f'{color.CVIOLET2}\n   -- removing Media cfg label {color.CYELLOW}{pathLabel}{color.CVIOLET2}' + \
                f' containing \n      {color.CWHITE2}{str(allItems[pathLabel])}{color.CEND}')

        self.remove_item(itemLabel = pathLabel)

    # --------------------------------------------------------------------------------------------------
    def get_configured_entries(self):
        allItems = self.get_all_configured_items()
        return allItems.keys()

    # --------------------------------------------------------------------------------------------------
    def get_color_label(self, pathLabel: str):
        allItems = self.get_all_configured_items()
        if pathLabel in allItems.keys():
            itemData = self.get_cfg_data_for_item_label(itemLabel = pathLabel)
        else:
            itemData = None
        if not itemData:
            return None, None
        return itemData['color'], colorTypeLkup[itemData['color']]

    # --------------------------------------------------------------------------------------------------
    def show_media_config(self):
        MultiEntityCoreCfg.show_full_config(self)

    # --------------------------------------------------------------------------------------------------
    def show_media_config_labels(self):
        MultiEntityCoreCfg.show_available_item_labels(self)

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------




#!/usr/bin/env python3
# ------------------------------------------------------------------------------------------------------
# -- CLI Methods for Search
# ------------------------------------------------------------------------------------------------------
# ======================================================================================================

# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse

import json
import os
import sys

from quickcolor.color_def import color
from showexception.showexception import exception_details

from media_mgr.server_cfg import ServerConfig
from media_mgr.media_cfg import MediaConfig

from media_mgr.search import show_titles_in_paths, show_matched_titles, show_all_matched_titles

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli():
    try:
        preMc = MediaConfig()

        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CYELLOW2}Search {color.CEND}for media manager',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        parser.add_argument('--version', action="store_true", help='top-level package version')

        subparsers = parser.add_subparsers(dest='cmd')

        p_showTitlesInPaths = subparsers.add_parser('show.titles.in.paths', help='show titles in one or more paths')
        p_showTitlesInPaths.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showTitlesInPaths.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')
        p_showTitlesInPaths.add_argument("paths", metavar='<path_1> .. <path_n>', nargs=argparse.REMAINDER,
                choices=preMc.get_configured_entries(),
                help='Search paths to probe')

        p_showMatchedTitles = subparsers.add_parser('show.match', help='show matched titles')
        p_showMatchedTitles.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')
        p_showMatchedTitles.add_argument('--type', default='plex',
                metavar='<srvType>', choices=['plex', 'worker'], help='Server types')
        p_showMatchedTitles.add_argument("terms", metavar='<term_1> .. <term_n>', nargs=argparse.REMAINDER,
                help='Search terms to match')

        p_showAllMatchedTitles = subparsers.add_parser('show.all.matches', help='show matched titles from all servers')
        p_showAllMatchedTitles.add_argument("terms", metavar='<term_1> .. <term_n>', nargs='*',
                help='Search terms to match')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.cmd == 'show.titles.in.paths':
            show_titles_in_paths(ipv4 = args.ipv4, serverType = args.type,
                    paths = args.paths, verbose = args.verbose)

        elif args.cmd == 'show.match':
            show_matched_titles(ipv4 = args.ipv4, serverType = args.type,
                    searchTerms = args.terms)

        elif args.cmd == 'show.all.matches':
            show_all_matched_titles(searchTerms = args.terms)

    except Exception as e:
        exception_details(e, "Search CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

def cli_search_plex():
    try:
        parser = argparse.ArgumentParser(
                    description=f'{"-." * 3}  {color.CBLUE2}Search {color.CYELLOW2}All Servers {color.CEND}utility',
                    epilog='-.' * 40)

        parser.add_argument('-v', '--verbose', action="store_true",
                            help='run with verbosity hooks enabled')

        parser.add_argument('--version', action='store_true', help='top-level package version')

        parser.add_argument('--ipv4', default=None, metavar='<ipv4.addr>', help='Server IPV4')

        parser.add_argument("terms", metavar='<term_1> .. <term_n>', nargs='*',
                help='Search terms to match')

        argcomplete.autocomplete(parser)
        args = parser.parse_args()
        # print(args)

        if len(sys.argv) == 1:
            parser.print_help(sys.stderr)
            sys.exit(1)

        if args.version:
            from importlib.metadata import version
            import media_mgr
            print(f'{color.CGREEN}{os.path.basename(sys.argv[0])}{color.CEND} resides in package ' + \
                    f'{color.CBLUE2}{media_mgr.__package__}{color.CEND} ' + \
                    f'version {color.CVIOLET2}{version("media_mgr")}{color.CEND} ...')
            sys.exit(0)

        if args.ipv4:
            srv = ServerConfig()
            for serverLabel in srv.get_server_name_list():
                if args.ipv4 == srv.get_server_address(serverLabel = serverLabel):
                    return show_matched_titles(ipv4 = args.ipv4,
                                               serverType = srv.get_server_type(serverLabel = serverLabel),
                                               searchTerms = args.terms,
                                               verbose = args.verbose)

            print(f'{color.CRED2}Error: {color.CYELLOW}{args.ipv4}{color.CEND} ' + \
                    f'is not registered as either a Plex or Worker server!')

        else:
            show_all_matched_titles(searchTerms = args.terms,
                                    verbose = args.verbose)

    except Exception as e:
        exception_details(e, "Search Plex/Worker Server(s) CLI")

# ------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------

